# Code for Beveridge data

library(assist)
library(tseries)

data(bev)
y <- log(bev)
x <- seq(0,1,len=length(y))

bev.fit1 <- ssr(y~x, rk=cubic(x))
summary(bev.fit1)

bev.fit2 <- update(bev.fit1, spar="m", correlation=corAR1(form=~1))
summary(bev.fit2)

p <- predict(bev.fit2)
year <- 1500:1869
postscript(file="beveridge.ps",height=2.8,width=4.4,
  horizontal=F,pointsize=8)
par(mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)
plot(year, y, ylab="log of price index", type="n")
polygon(c(year,rev(year)),
 c(p$fit-1.96*p$pstd,rev(p$fit+1.96*p$pstd)),
 col=gray(0:8/8)[8], border=NA)
points(year, y, pch="o", cex=.4)
lines(year, bev.fit1$fit, lty=2)
lines(year, p$fit, lwd=1.5)
dev.off()


