# Code for the simulation in section "Vector Spline"

library(assist)

# generate paired data
set.seed(2353)
n <- 100
s1 <- .5
s2 <- 1
r <- .8
A <- diag(c(s1,s2))%*%matrix(c(sqrt(1-r**2),0,r,1),2,2)
e <- NULL
for (i in 1:n) e <- c(e,A%*%rnorm(2))
x <- 1:n/n
y1 <- sin(2*pi*x) + e[seq(1,2*n,by=2)]
y2 <- sin(2*pi*x) + 2*x + e[seq(2,2*n,by=2)]
bisp.dat <- data.frame(y=c(y1,y2),x=rep(x,2),
 id=as.factor(rep(c(0,1),rep(n,2))), pair=rep(1:n,2))

# fit separately
bisp.fit1 <- ssr(y~I(x-.5), rk=cubic(x), spar="m",
 data=bisp.dat[bisp.dat$id==0,])
bisp.p1 <- predict(bisp.fit1)
bisp.fit2 <- ssr(y~I(x-.5), rk=cubic(x), spar="m",
 data=bisp.dat[bisp.dat$id==1,])
bisp.p2 <- predict(bisp.fit2)

# fit jointly	
#bisp.fit3 <- ssr(y~id*I(x-.5), 
#        rk=list(rk.prod(cubic(x),kron(id==0)),
#                rk.prod(cubic(x),kron(id==1))), spar="m", 
#                weights=varIdent(form=~1|id),
#                correlation=corSymm(form=~1|pair), 
#                data=bisp.dat)

bisp.fit3 <- ssr(y~id/I(x-.5)-1, 
        rk=list(rk.prod(cubic(x),kron(id==0)),
                rk.prod(cubic(x),kron(id==1))), spar="m", 
                weights=varIdent(form=~1|id),
                correlation=corSymm(form=~1|pair), 
                data=bisp.dat)
summary(bisp.fit3)

# f1
bisp.p31 <- predict(bisp.fit3, newdata=bisp.dat[bisp.dat$id==0,], terms=c(1,0,1,0,1,0))
# f2
bisp.p32 <- predict(bisp.fit3, newdata=bisp.dat[bisp.dat$id==1,], terms=c(0,1,0,1,0,1))
mean((bisp.p1$pstd-bisp.p31$pstd)/bisp.p1$pstd)
mean((bisp.p2$pstd-bisp.p32$pstd)/bisp.p2$pstd)

postscript(file="bisp1.ps",height=4,width=4.4,horizontal=F,pointsize=9)
par(mfrow=c(2,2), mgp=c(2,1,0), mar=c(3,3,3,1)+0.1)	
plot(x, bisp.p1$fit, type="n", ylab=expression(f[1]), ylim=c(-1.7,1.7))
polygon(c(x,rev(x)),
        c(bisp.p1$fit-1.96*bisp.p1$pstd,
          rev(bisp.p1$fit+1.96*bisp.p1$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(x, sin(2*pi*x), lty=3)
lines(x, bisp.p1$fit)
mtext(expression(paste("separate estimate of ",f[1])), cex=1.1)

plot(x, bisp.p2$fit, type="n", ylab=expression(f[2]), ylim=c(-1,3))
polygon(c(x,rev(x)),
        c(bisp.p2$fit-1.96*bisp.p2$pstd,
          rev(bisp.p2$fit+1.96*bisp.p2$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(x, sin(2*pi*x)+2*x, lty=3)
lines(x, bisp.p2$fit)
mtext(expression(paste("separate estimate of ",f[2])), cex=1.1)

plot(x, bisp.p31$fit, type="n", ylab=expression(f[1]), ylim=c(-1.7,1.7))
polygon(c(x,rev(x)),
        c(bisp.p31$fit-1.96*bisp.p31$pstd,
          rev(bisp.p31$fit+1.96*bisp.p31$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(x, sin(2*pi*x), lty=3)
lines(x, bisp.p31$fit)
mtext(expression(paste("joint estimate of ",f[1])), cex=1.1)

plot(x, bisp.p32$fit, type="n", ylab=expression(f[2]), ylim=c(-1,3))
polygon(c(x,rev(x)),
        c(bisp.p32$fit-1.96*bisp.p32$pstd,
          rev(bisp.p32$fit+1.96*bisp.p32$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(x, sin(2*pi*x)+2*x, lty=3)
lines(x, bisp.p32$fit)
mtext(expression(paste("joint estimate of ",f[2])), cex=1.1)
dev.off()

bisp.fit4 <- update(bisp.fit3,
 y~I(x-.5)+I(id==1)+I((x-.5)*(id==1)), 
 rk=list(cubic(x),rk.prod(cubic(x),kron(id==1))))
summary(bisp.fit4)

# f2-f1
bisp.p41 <- predict(bisp.fit4, newdata=bisp.dat[bisp.dat$id==1,], terms=c(0,0,1,1,0,1))
# f2-f1 w/o constant
bisp.p42 <- predict(bisp.fit4, newdata=bisp.dat[bisp.dat$id==1,], terms=c(0,0,0,1,0,1))
# f2-f1 w/o constant and linear
bisp.p43 <- predict(bisp.fit4, newdata=bisp.dat[bisp.dat$id==1,], terms=c(0,0,0,0,0,1))

	
postscript(file="bisp2.ps",height=2,width=4.4,horizontal=F,pointsize=9)
par(mfrow=c(1,3), mgp=c(2,1,0), mar=c(3,3,3,1)+0.1)	
plot(x, bisp.p41$fit, type="n", ylab="d", ylim=c(-1,3))
polygon(c(x,rev(x)),
        c(bisp.p41$fit-1.96*bisp.p41$pstd,
          rev(bisp.p41$fit+1.96*bisp.p41$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(x, 2*x, lty=3)
lines(x, bisp.p41$fit)
mtext("estimate of d(x)", cex=1.1)

plot(x, bisp.p42$fit, type="n", ylab=expression(d[1]), ylim=c(-2,2))
polygon(c(x,rev(x)),
        c(bisp.p42$fit-1.96*bisp.p42$pstd,
          rev(bisp.p42$fit+1.96*bisp.p42$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(x, 2*x-1, lty=3)
lines(x, bisp.p42$fit)
mtext(expression(paste("estimate of ", d[1], "(x)")), cex=1.1)

plot(x, bisp.p43$fit, type="n", ylab=expression(d[2]), ylim=c(-1e-4,1e-4))
polygon(c(x,rev(x)),
        c(bisp.p43$fit-1.96*bisp.p43$pstd,
          rev(bisp.p43$fit+1.96*bisp.p43$pstd)),
          col=gray(0:8/8)[8], border=NA)
abline(0, 0, lty=3)
lines(x, bisp.p43$fit)
mtext(expression(paste("estimate of ", d[2], "(x)")), cex=1.1)
dev.off()	
			
