# Code for bond data

#creat integral.so file by  "> R CMD SHLIB integral.c"
library(assist)
dyn.load("integral.so")
source("int.r")
source("one.bond.r")
source("two.bond.r")
data(bond); attach(bond)
nboot <- 5000

# bootstraping for separate fit
boot.bond.one <- function(x, y, yhat, X, T, Q, 
 spar="m", limnla=c(-3,6), grid, nboot, seed=0) {
 set.seed <- seed
 resi <- y-yhat
 gb <- NULL
 for(i in 1:nboot) {
  yb <- yhat + sample(resi, replace=TRUE)
  tmp <- try(ssr(yb~T-1, Q, spar=spar, limnla=c(-3,6)))
  if(class(tmp)!="try-error") gb <- cbind(gb, 
   cbind(1,grid)%*%tmp$coef$d+cubic2(grid,x)%*%t(X)%*%tmp$coef$c)
 }
 return(gb)
}

# bootstraping for joint fit
boot.bond.two <- function(y, yhat, price, payment, time, name,
 type, spar="m", limnla=c(-3,6), nboot, seed=0) {
 set.seed <- seed
 resi <- y-yhat
 group <- c(as.vector(table(name[type=="govt"])),
            as.vector(table(name[type=="ge"])))
 fb <- f2b <- gb <- NULL
 for(i in 1:nboot) {
  price.b <- rep(yhat+sample(resi,replace=TRUE), group)
  tmp <- try(two.bond(price=price.b, payment=payment, time=time,
                      name=name, type=type))
  if(class(tmp)!="try-error"&tmp$iter[2]<1.e-4&tmp$iter[3]==0) {
   fb <- cbind(fb, c(tmp$f.val$f1,tmp$f.val$f2))
   f2b <- cbind(f2b, tmp$f2.val)
   gb <- cbind(gb, c(tmp$dc[[1]],tmp$dc[[2]]))
  }
  list(fb=fb, f2b=f2b, gb=gb)
 }
}

# fit discount function directly
group <- as.vector(table(name[type=="govt"]))
y <- price[type=="govt"][cumsum(group)]
x <- time[type=="govt"]
r <- payment[type=="govt"]
X <- assist:::diagComp(matrix(r,nrow=1),group)
S <- cbind(1,x)
T <- X%*%S
Q <- X%*%cubic2(x)%*%t(X)
bond.cub.fit.govt <- ssr(y~T-1, Q, spar="m", limnla=c(6,10))
grid1 <- seq(min(x),max(x),len=100)
g.cub.p.govt <- cbind(1,grid1)%*%bond.cub.fit.govt$coef$d+
 cubic2(grid1,x)%*%t(X)%*%bond.cub.fit.govt$coef$c
g.cub.b.govt <- boot.bond.one(x,y,yhat=bond.cub.fit.govt$fit,
  X,T,Q,grid=grid1,nboot=nboot,seed=3498) 

group <- as.vector(table(name[type=="ge"]))
y <- price[type=="ge"][cumsum(group)]
x <- time[type=="ge"]
r <- payment[type=="ge"]
X <- assist:::diagComp(matrix(r, nrow=1),group)
S <- cbind(1, x)
T <- X%*%S
Q <- X%*%cubic2(x)%*%t(X)
bond.cub.fit.ge <- ssr(y~T-1, Q, spar="m")
grid2 <- seq(min(x),max(x),len=100)
g.cub.p.ge <- cbind(1,grid2)%*%bond.cub.fit.ge$coef$d+
 cubic2(grid2,x)%*%t(X)%*%bond.cub.fit.ge$coef$c
g.cub.b.ge <- boot.bond.one(x,y,yhat=bond.cub.fit.ge$fit,
  X,T,Q,spar="m",grid=grid2,nboot=nboot,seed=2309) 

# fit separate nnr models 
bond.nnr.fit.govt <- one.bond(price= price[type=="govt"],
 payment=payment[type=="govt"], time=time[type=="govt"],
 name=name[type=="govt"], spar="m")

bond.nnr.fit.ge <- one.bond(price= price[type=="ge"], 
 payment=payment[type=="ge"], time=time[type=="ge"],
 name=name[type=="ge"], spar="m")

# fit a joint nnr model 
bond.nnr.fit <- two.bond(price=price, payment=payment, time=time,
 name=name, type=type, spar="m")

gf.nnr.b <- boot.bond.two(y=bond.nnr.fit$y$y,
 yhat=bond.nnr.fit$y$yhat, price=price, payment=payment, time=time,
 name=name, type=type, nboot=nboot, seed=2394)

postscript(file="bond.ps",height=4,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(2,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(grid1, g.cub.p.govt, xlab="time (yr)", ylab="discount rate", 
     type="n", ylim=c(.1,1.1))
mtext("(a)")
gl <- apply(g.cub.b.govt,1,quantile,prob=.025)
gu <- apply(g.cub.b.govt,1,quantile,prob=.975)
# bounds are too narrow to show
polygon(c(grid1,rev(grid1)),c(gl,rev(gu)),
 col=gray(0:8/8)[8], border=NA)
gl <- apply(g.cub.b.ge,1,quantile,prob=.025)
gu <- apply(g.cub.b.ge,1,quantile,prob=.975)
polygon(c(grid2,rev(grid2)),c(gl,rev(gu)),
 col=gray(0:8/8)[8], border=NA)
lines(grid1, g.cub.p.govt)
lines(grid2, g.cub.p.ge, lty=2)

year1 <- time[type=="govt"]
year2 <- time[type=="ge"]
n1 <- nrow(bond[type=="govt",])
plot(sort(year1), bond.nnr.fit$dc[[1]][order(year1)],type="n",
     xlab="time (yr)", ylab="discount rate", ylim=c(.1,1.1))
mtext("(b)")
gl <- apply(gf.nnr.b$gb[1:n1,],1,quantile,prob=.025)
gu <- apply(gf.nnr.b$gb[1:n1,],1,quantile,prob=.975)
polygon(c(sort(year1),rev(sort(year1))),
 c(gl[order(year1)],rev(gu[order(year1)])),
 col=gray(0:8/8)[8], border=NA)
gl <- apply(gf.nnr.b$gb[-c(1:n1),],1,quantile,prob=.025)
gu <- apply(gf.nnr.b$gb[-c(1:n1),],1,quantile,prob=.975)
polygon(c(sort(year2),rev(sort(year2))),
 c(gl[order(year2)],rev(gu[order(year2)])),
 col=gray(0:8/8)[8], border=NA)
lines(sort(year1), bond.nnr.fit$dc[[1]][order(year1)])
lines(sort(year2), bond.nnr.fit$dc[[2]][order(year2)], lty=2)

x1 <- bond.nnr.fit$q.time[[1]]
plot(x1, bond.nnr.fit$f.val[[1]],type="n",
     xlab="time (yr)", ylab="forward rate", ylim=c(0,.1))
mtext("(c)")
gl <- apply(gf.nnr.b$fb[1:length(x1),],1,quantile,prob=.025)
gl[gl<0] <- 0
gu <- apply(gf.nnr.b$fb[1:length(x1),],1,quantile,prob=.975)
polygon(c(x1,rev(x1)), c(gl,rev(gu)), col=gray(0:8/8)[8], border=NA)
x2 <- bond.nnr.fit$q.time[[2]]
gl <- apply(gf.nnr.b$fb[-c(1:length(x1)),],1,quantile,prob=.025)
gl[gl<0] <- 0
gu <- apply(gf.nnr.b$fb[-c(1:length(x1)),],1,quantile,prob=.975)
polygon(c(x2,rev(x2)), c(gl,rev(gu)), col=gray(0:8/8)[8], border=NA)
lines(x1, bond.nnr.fit$f.val[[1]])
lines(x2, bond.nnr.fit$f.val[[2]], lty=2)

x <- bond.nnr.fit$q.time[[2]]
plot(x, bond.nnr.fit$f2.val, type="n", ylim=c(-.025,.02),
      xlab="time (yr)", ylab="credit spread")
mtext("(d)")
gl <- apply(gf.nnr.b$f2b,1,quantile,prob=.025)
gu <- apply(gf.nnr.b$f2b,1,quantile,prob=.975)
polygon(c(x,rev(x)), c(gl,rev(gu)), col=gray(0:8/8)[8], border=NA)
lines(x, bond.nnr.fit$f2.val)

dev.off()




