# Code for chickenpox data

library(assist)
library(lattice)

data(chickenpox)
y <- sqrt(chickenpox$count) 
x1 <- (chickenpox$month-0.5)/12 
x2 <- ident(chickenpox$year) 

tmp <- ssr(y~1, rk=periodic(x1), spar="m")
g3.ini <- predict(tmp, term=c(0,1))$fit
chick.nnr <- nnr(y~g1(x2)+exp(g2(x2))*g3(x1), 
            func=list(g1(x)~list(~I(x-.5),cubic(x)), 
                      g2(x)~list(~I(x-.5)-1,cubic(x)), 
                      g3(x)~list(~sin(2*pi*x)+cos(2*pi*x)-1,lspline(x,type="sine0"))), 
            start=list(g1=mean(y),g2=0,g3=g3.ini), 
            control=list(converg="coef"),spar="m")

grid <- data.frame(x1=seq(0,1,len=50), x2=seq(0,1,len=50)) 
chick.nnr.pred <- intervals(chick.nnr, newdata=grid, 
                 terms=list(g1=matrix(c(1,1,1,1,1,0,0,0,1),nrow=3,byrow=T), 
	                    g2=matrix(c(1,1,1,0,0,1),nrow=3,byrow=T), 
                            g3=matrix(c(1,1,1,1,1,0,0,0,1),nrow=3,byrow=T))) 
 
mr <- diff(range(chick.nnr.pred$g3$fit[,1])) 
yr <- apply(matrix(y/mr, nrow=12, byrow=F), 2, function(x) diff(range(x))) 
ya <- apply(matrix(y, nrow=12, byrow=F), 2, mean) 
postscript(file="chickenpox1.ps",height=2.1,width=4.4,horizontal=F,pointsize=7) 
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,3,1)+0.1) 
plot(grid$x2, chick.nnr.pred$g1$fit[,1], xlab="year", ylab=expression(g[1]), 
     ylim=c(15,30), type="l",axes=F) 
box() 
axis(2,cex=.6) 
axis(1,at=seq(0,41,by=10)/41, label=as.character(seq(31,71,by=10)),cex=0.6) 
polygon(c(grid$x2,rev(grid$x2)),
        c(chick.nnr.pred$g1$fit[,1]-1.96*chick.nnr.pred$g1$pstd[,1],
          rev(chick.nnr.pred$g1$fit[,1]+1.96*chick.nnr.pred$g1$pstd[,1])),
        col="lightgray",border=NA)
points((0:41)/42,ya,pch="o",cex=.6) 
lines(grid$x2, chick.nnr.pred$g1$fit[,1]) 
mtext("(a)")

plot(grid$x2, chick.nnr.pred$g2$fit[,1], xlab="year", ylab=expression(g[2]), 
     ylim=c(-.8,.4), type="l",axes=F) 
box() 
axis(2,cex=.6) 
axis(1,at=seq(0,41,by=10)/41, label=as.character(seq(31,71,by=10)),cex=0.6) 
polygon(c(grid$x2,rev(grid$x2)),
        c(chick.nnr.pred$g2$fit[,1]-1.96*chick.nnr.pred$g2$pstd[,1],
          rev(chick.nnr.pred$g2$fit[,1]+1.96*chick.nnr.pred$g2$pstd[,1])),
        col="lightgray",border=NA)
points((0:41)/42,log(yr)-mean(log(yr)),pch="o",cex=.6) 
lines(grid$x2, chick.nnr.pred$g2$fit[,1]) 
mtext("(b)")
dev.off() 
	 
x <- grid$x1*12+.5
fit.data <- data.frame(y=c(chick.nnr.pred$g3$fit[,1], chick.nnr.pred$g3$fit[,2],
  chick.nnr.pred$g3$fit[,3]),
  x=rep(x, 3),
  grp=rep(c("overall","parametric", "smooth"), rep(50, 3)))
low.data <- data.frame(y=c(chick.nnr.pred$g3$fit[,1]-1.96*chick.nnr.pred$g3$pstd[,1],
  chick.nnr.pred$g3$fit[,2]-1.96*chick.nnr.pred$g3$pstd[,2],
  chick.nnr.pred$g3$fit[,3]-1.96*chick.nnr.pred$g3$pstd[,3]),
  x=rep(x, 3),
  grp=rep(c("overall","parametric", "smooth"), rep(50, 3)))
up.data <- data.frame(y=c(chick.nnr.pred$g3$fit[,1]+1.96*chick.nnr.pred$g3$pstd[,1],
  chick.nnr.pred$g3$fit[,2]+1.96*chick.nnr.pred$g3$pstd[,2],
  chick.nnr.pred$g3$fit[,3]+1.96*chick.nnr.pred$g3$pstd[,3]),
  x=rep(x, 3),
  grp=rep(c("overall","parametric", "smooth"), rep(50, 3)))

trellis.device(postscript, file="chickenpox2.ps", width=5.72,
               height=2.6, horizontal=F)
xyplot2(y~x|grp, data=list(fit.data, up.data, low.data), layout=c(3,1,1),
  par.strip.text = list(cex = 0.75),
  xlab=list(label="month", cex=.8),
  ylab=list(label=expression(g[3]), cex=.8))
dev.off()


#full SS ANOVA model
chick.ssanova <- ssr(y~x2, 
          rk=list(periodic(x1),cubic(x2), 
                  rk.prod(periodic(x1),kron(x2-.5)), 
                  rk.prod(periodic(x1),cubic(x2))))

n <- length(y) 
rss <- c(sum(chick.ssanova$resi**2),sum(chick.nnr$resi**2))/n
df <- c(chick.ssanova$df, chick.nnr$df$f)
gcv <- rss/(1-df/n)**2
aic <- n*log(rss)+2*df
bic <- n*log(rss)+log(n)*df
print(round(rbind(gcv,aic,bic),2))

# plot of time series and overall fits
t <- chickenpox$month+(chickenpox$year-1931)*12 
postscript(file="chickenpox3.ps",height=2.5,width=4.4,
  horizontal=F,pointsize=7) 
plot(t, y, xlab="year", ylab="y", ylim=c(5,60), type="l",axes=F,lty=3) 
box() 
axis(2,cex=.6) 
axis(1,at=seq(6,498,by=120), label=as.character(seq(31,71,by=10)),cex=0.6) 
lines(t, y-chick.nnr$resi, lty=1)
lines(t, chick.ssanova$fit, lty=2)
dev.off()


