# Simulations code for confidence intervasl

library(assist)

set.seed(3874)
n <- 100
s <- .1
x <- 1:n/n
#f <- sin(4*pi*x**2)
f <- exp(-(x-.5)**2*64)
nsimu <- 100
nboot <- 1000

fest <- matrix(NA,n,nsimu)
ci.bayes <- ci.boot.per <- ci.boot.t <- array(NA,c(n,nsimu,2))
for (i in 1:nsimu) {
  y <- f+s*rnorm(n)
  fit <- ssr(y~x,cubic(x),spar="v")
  pred <- predict(fit)
  fest[,i] <- pred$fit
  ci.bayes[,i,1] <- pred$fit-1.96*pred$pstd
  ci.bayes[,i,2] <- pred$fit+1.96*pred$pstd

  fhat <- fit$fit
  shat <- fit$sigma
  fhatb <- d <- NULL
  for (j in 1:nboot) {
    yb <- fhat+shat*rnorm(n)
    tmp <- ssr(yb~x,cubic(x),spar="v")
    fhatb <- cbind(fhatb,tmp$fit)
    d <- cbind(d, (tmp$fit-fhat)/tmp$sigma)
  }
  ci.boot.per[,i,1] <- apply(fhatb,1,quantile,prob=.025) 
  ci.boot.per[,i,2] <- apply(fhatb,1,quantile,prob=.975) 
  ci.boot.t[,i,1] <- fhat-shat*apply(d,1,quantile,prob=.975) 
  ci.boot.t[,i,2] <- fhat-shat*apply(d,1,quantile,prob=.025) 
}

#bs <- apply((fest-f)**2, 1, mean)
#v <- apply(fest, 1, var)

pcov.bayes <- apply(ci.bayes[,,1]<=f & ci.bayes[,,2]>=f,1,mean)
pcov.boot.per <- apply(ci.boot.per[,,1]<=f & ci.boot.per[,,2]>=f,1,mean)
pcov.boot.t <- apply(ci.boot.t[,,1]<=f & ci.boot.t[,,2]>=f,1,mean)

acp.bayes <- apply(ci.bayes[,,1]<=f & ci.bayes[,,2]>=f,2,mean)
acp.boot.per <- apply(ci.boot.per[,,1]<=f & ci.boot.per[,,2]>=f,2,mean)
acp.boot.t <- apply(ci.boot.t[,,1]<=f & ci.boot.t[,,2]>=f,2,mean)

postscript(file="simuci.ps",height=3.5,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(2,2), mgp=c(2,1,0), mar=c(3,3,2,1)+0.1)

ddf <- 2*f*(2*64**2*(x-.5)**2-64)
plot(x,pcov.bayes,ylim=c(0,1),ylab="pointwise coverage",type="l")
lines(x,abs(ddf)/(1.5*max(abs(ddf))),lty=2)
abline(.95,0,lty=3)
mtext("(a)")

plot(x,pcov.boot.per,ylim=c(0,1),ylab="pointwise coverage",type="l")
lines(x,abs(ddf)/(1.5*max(abs(ddf))),lty=2)
abline(.95,0,lty=3)
mtext("(b)")

plot(x,pcov.boot.t,ylim=c(0,1),ylab="pointwise coverage",type="l")
lines(x,abs(ddf)/(1.5*max(abs(ddf))),lty=2)
abline(.95,0,lty=3)
mtext("(c)")

bp <- boxplot(split(c(acp.bayes,acp.boot.per,acp.boot.t),
  rep(1:3,rep(nsimu,3))), names=c("Bayes","Per","T"),
  ylab="average coverage")
xi <- .3+seq(bp$n)
points(seq(bp$n),c(mean(acp.bayes),mean(acp.boot.per),mean(acp.boot.t)),  
  pch="+",cex=1.5)
abline(.95,0,lty=3)  
mtext("(d)")

dev.off()




