# Code for dog data

library(assist)
library(lattice)
data(dog)

# spaghetti plot
postscript("dog1.ps", pointsize=9, width=4.4,height=4,horizontal=F)
par(mfrow=c(2,2), mgp=c(2,1,0), mar=c(3,3,1,1)+.1)
a <- seq(1,13,by=2)
for (j in 1:4) {
    b <- sapply(split(dog$y[dog$group==j],
	 dog$time[dog$group==j]),mean)
    plot(a,b,xlab="time (min)", ylab="potassium", type="n",
	 ylim=range(dog$y),xaxt="n")
    axis(side=1,at=a)
    lines(a,b,lwd=3)
    for (i in 1:(sum(dog$group==j)/7))
       lines(a,dog$y[dog$group==j][(1+(i-1)*7):(i*7)],type="b",
	     pch=as.character(i-1),cex=.6)
    mtext(paste("Group ",as.character(j)))
}
dev.off()

# fit Model 1
dog.fit1 <- slm(y~time, rk=list(cubic(time), shrink1(group), 
                rk.prod(kron(time-.5), shrink1(group)),
                rk.prod(cubic(time), shrink1(group))), 
                random=list(dog=~1), data=dog)
summary(dog.fit1)

# fit Model 2
dog.fit2 <- update(dog.fit1, random=list(dog=~time))
summary(dog.fit2)

# fit Model 3
D2 <- cubic(dog$time[1:7])
H <- chol.new(D2)
Z <- kronecker(diag(36), H)
dog$all <- rep(1,36*7)
dog.fit3 <- update(dog.fit2, random=list(all=pdIdent(~Z-1), dog=~time)) 
summary(dog.fit3)

# reduce dimension
chol.new1 <- function(Q, cutoff) {
  tmp <- eigen(Q)
  num <- sum(tmp$values < cutoff)
  k <- ncol(Q) - num
#  assist:::matDiag.prod(as.matrix(tmp$vector[, 1:k]), sqrt(tmp$values[1:k]),  FALSE)
 t(t(as.matrix(tmp$vector[,1:k]))*sqrt(tmp$values[1:k]))
}
H1 <- chol.new1(D2, 1e-3)
Z1 <- kronecker(diag(36), H1)
dog$all1 <- rep(1,nrow(Z1))
dog.fit3.1 <- update(dog.fit2, random=list(all1=pdIdent(~Z1-1), dog=~time))
summary(dog.fit3.1)

# low rank approximation for fixed smooth components too
Q1 <- cubic(dog$time)
Q2 <- shrink1(dog$group)
Q3 <- rk.prod(kron(dog$time-.5), Q2)
Q4 <- rk.prod(Q1, Q2)
Z2 <- chol.new1(Q1, 1e-4)
Z3 <- chol.new1(Q2, 1e-4)
Z4 <- chol.new1(Q3, 1e-4)
Z5 <- chol.new1(Q4, 1e-4)
dog$all2 <- dog$all3 <- dog$all4 <- dog$all5 <- rep(1,nrow(Z1))
dog.fit3.2 <- lme(y~time, data=dog, 
 random=list(all2=pdIdent(~Z2-1), all3=pdIdent(~Z3-1),
             all4=pdIdent(~Z4-1), all5=pdIdent(~Z5-1),
             all1=pdIdent(~Z1-1), dog=~time))
dog.grid <- data.frame(time=rep(seq(0,1,len=50),4),
                       group=as.factor(rep(1:4,rep(50,4))),
                       all1=rep(1,200))
R1 <- cubic(dog.grid$time,dog$time)
R2 <- shrink1(dog.grid$group,dog$group)
R3 <- rk.prod(kron(dog.grid$time-.5,dog$time-.5), R2)
R4 <- rk.prod(R1, R2)
Z2.qr <- qr(Z2)
Z3.qr <- qr(Z3)
Z4.qr <- qr(Z4)
Z5.qr <- qr(Z5)
r2 <- qr.Q(Z2.qr)%*%solve(t(qr.R(Z2.qr)))%*%as.vector(dog.fit3.2$coef$random$all2)
r3 <- qr.Q(Z3.qr)%*%solve(t(qr.R(Z3.qr)))%*%as.vector(dog.fit3.2$coef$random$all3)
r4 <- qr.Q(Z4.qr)%*%solve(t(qr.R(Z4.qr)))%*%as.vector(dog.fit3.2$coef$random$all4)
r5 <- qr.Q(Z5.qr)%*%solve(t(qr.R(Z5.qr)))%*%as.vector(dog.fit3.2$coef$random$all5)
r2 <- as.vector(R1%*%r2)
r3 <- as.vector(R2%*%r3)
r4 <- as.vector(R3%*%r4)
r5 <- as.vector(R4%*%r5)
e.dog.fit3.2 <- dog.fit3.2$coef$fixed[1]+dog.fit3.2$coef$fixed[2]*dog.grid$time+
 r2+r3+r4+r5

trellis.device(postscript, file="doglowrank.ps", color=F, width=4.6, height=2.2,
               horizontal=F, theme=list(fontsize=list(text=8, points=4)))
tmp1 <- data.frame(y=e.dog.fit3.2, time=dog.grid$time*12+1, group=dog.grid$group)
tmp2 <- data.frame(time=rep(seq(1,13,by=2),4), group=rep(1:4,rep(7,4)))
tmp <- matrix(dog$y, nrow=7, byrow=F)
tmp2$y <- c(as.vector(apply(tmp[,1:9],  1,mean)),
            as.vector(apply(tmp[,10:19],1,mean)),
            as.vector(apply(tmp[,20:27],1,mean)),
            as.vector(apply(tmp[,28:36],1,mean)))
tmp1$group <- factor(tmp1$group, levels=c(1,2,3,4))
xyplot2(y~time|group, data=list(tmp1,tmp2), type=c("l","p"),
        xlab="time (min)", ylab="",layout=c(4,1))
dev.off()



# compare three corresponding LME models
anova(dog.fit1$lme.obj, dog.fit2$lme.obj, dog.fit3$lme.obj)

# compute population mean curves from Model 3
dog.grid <- data.frame(time=rep(seq(0,1,len=50),4), 
                       group=as.factor(rep(1:4,rep(50,4))))
e.dog.fit3 <- intervals(dog.fit3, newdata=dog.grid, terms=rep(1,6))

trellis.device(postscript, file="dog2.ps", color=F, width=4.6, height=2.2, 
               horizontal=F, theme=list(fontsize=list(text=8, points=4)))
tmp1 <- data.frame(y=e.dog.fit3$fit, time=dog.grid$time*12+1, group=dog.grid$group)
tmp2 <- tmp3 <- tmp1 
tmp2$y <- tmp2$y+1.96*e.dog.fit3$pstd
tmp3$y <- tmp3$y-1.96*e.dog.fit3$pstd
tmp4 <- data.frame(time=rep(seq(1,13,by=2),4), group=rep(1:4,rep(7,4)))
tmp <- matrix(dog$y, nrow=7, byrow=F)
tmp4$y <- c(as.vector(apply(tmp[,1:9],  1,mean)), 
            as.vector(apply(tmp[,10:19],1,mean)),
            as.vector(apply(tmp[,20:27],1,mean)), 
            as.vector(apply(tmp[,28:36],1,mean)))
tmp1$group <- factor(tmp1$group, levels=c(1,2,3,4))
xyplot2(y~time|group, data=list(tmp1,tmp2,tmp3,tmp4), type=c("l","l","l","p"), 
        xlab="time (min)", ylab="",layout=c(4,1))
dev.off()

# fit Model 1 under new penalty
dog.fit4 <- slm(y~group*time, rk=list(rk.prod(cubic(time), kron(group==1)),
                                      rk.prod(cubic(time), kron(group==2)),
                                      rk.prod(cubic(time), kron(group==3)),
                                      rk.prod(cubic(time), kron(group==4))), 
	      random=list(dog=~1), data=dog)
dog.fit4

dog.fit5 <- update(dog.fit4, random=list(dog=~time))
dog.fit5

dog.fit6 <- update(dog.fit5, random=list(all=pdIdent(~Z-1), dog=~time)) 
dog.fit6
anova(dog.fit4$lme.obj, dog.fit5$lme.obj, dog.fit6$lme.obj)
dog.grid <- data.frame(time=rep(seq(0,1,len=50),4), 
                       group=as.factor(rep(1:4,rep(50,4))))
e.dog.fit6 <- intervals(dog.fit6, newdata=dog.grid, terms=rep(1,12))

trellis.device(postscript, file="dog3.ps", color=F, width=4.6, height=2.2, 
               horizontal=F, theme=list(fontsize=list(text=8, points=4)))
tmp1 <- data.frame(y=e.dog.fit6$fit, time=dog.grid$time*12+1, group=dog.grid$group)
tmp2 <- tmp3 <- tmp1 
tmp2$y <- tmp2$y+1.96*e.dog.fit6$pstd
tmp3$y <- tmp3$y-1.96*e.dog.fit6$pstd
tmp4 <- data.frame(time=rep(seq(1,13,by=2),4), group=rep(1:4,rep(7,4)))
tmp <- matrix(dog$y, nrow=7, byrow=F)
tmp4$y <- c(as.vector(apply(tmp[,1:9],  1,mean)), 
            as.vector(apply(tmp[,10:19],1,mean)),
            as.vector(apply(tmp[,20:27],1,mean)), 
            as.vector(apply(tmp[,28:36],1,mean)))
tmp1$group <- factor(tmp1$group, levels=1:4)
xyplot2(y~time|group, data=list(tmp1,tmp2,tmp3,tmp4), type=c("l","l","l","p"), 
        xlab="time (min)", ylab="",layout=c(4,1))
dev.off()
	
# prediction for all dogs
dog.grid2 <- data.frame(time=rep(seq(0,1,len=50),36),dog=rep(1:36,rep(50,36)))
R <- kronecker(diag(36),cubic(dog.grid2$time[1:50],dog$time[1:7]))	
b1 <- dog.fit6$lme.obj$coef$random$dog
b2 <- as.vector(dog.fit6$lme.obj$coef$random$all)
Z.qr <- qr(Z)
r <- qr.Q(Z.qr)%*%solve(t(qr.R(Z.qr)))%*%b2
tmp1 <- c(rep(e.dog.fit6$fit[dog.grid$group==1],9),
	  rep(e.dog.fit6$fit[dog.grid$group==2],10),
          rep(e.dog.fit6$fit[dog.grid$group==3],8),
          rep(e.dog.fit6$fit[dog.grid$group==4],9))
tmp2 <- as.vector(rep(b1[,1],rep(50,36)))
tmp3 <- as.vector(kronecker(b1[,2],dog.grid2$time[1:50]))	
u.new <- as.vector(R%*%r)
p.dog.fit6 <- tmp1+tmp2+tmp3+u.new

trellis.device(postscript, file="dog4.ps", color=F, width=4.6, height=2.2, 
               horizontal=F, theme=list(fontsize=list(text=8, points=4)))
tmp1 <- data.frame(y=dog$y[dog$dog==1|dog$dog==2|dog$dog==26|dog$dog==27], 
                time=dog$time[1:28]*12+1, group=rep(c("1","2","26","27"),rep(7,4)))
tmp2 <- data.frame(y=p.dog.fit6[dog.grid2$dog==1|dog.grid2$dog==2|dog.grid2$dog==26
                 |dog.grid2$dog==27], time=12*dog.grid2$time[1:200]+1, 
                 group=rep(c("1","2","26","27"),rep(50,4)))
xyplot2(y~time|group, data=list(tmp2,tmp1), type=c("l","p"), 
        xlab="time (min)", ylab="",layout=c(4,1))
dev.off()

# check prediction using prediction function
tmpfit <- predict(dog.fit6) [,3]
trellis.device(postscript, file="dog5.ps", color=F, width=4.6, height=2.2, 
               horizontal=F, theme=list(fontsize=list(text=8, points=4)))
tmp1 <- data.frame(y=dog$y[dog$dog==1|dog$dog==2|dog$dog==26|dog$dog==27], 
                time=dog$time[1:28]*12+1, group=rep(c("1","2","26","27"),rep(7,4)))
tmp2 <- data.frame(y=p.dog.fit6[dog.grid2$dog==1|dog.grid2$dog==2|dog.grid2$dog==26
                 |dog.grid2$dog==27], time=12*dog.grid2$time[1:200]+1, 
                 group=rep(c("1","2","26","27"),rep(50,4)))
tmp3 <- data.frame(y=tmpfit[dog$dog==1|dog$dog==2|dog$dog==26|dog$dog==27],
                time=dog$time[1:28]*12+1, group=rep(c("1","2","26","27"),rep(7,4)))

xyplot2(y~time|group, data=list(tmp2,tmp1,tmp3), type=c("l","p","l"), 
        xlab="time (min)", ylab="",layout=c(4,1))
dev.off()
		
		
		
	

