# Code for geyser data

library(assist)
attach(faithful)

# fit a straight line and check residuals
postscript(file="geyser1.ps",height=2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(eruptions,waiting,xlab="duration (min)",ylab="waiting (min)",type="n")
points(eruptions,waiting,pch=".")
geyser.para.fit <- lm(waiting~eruptions)
lines(sort(eruptions),geyser.para.fit$coef[1]+
      geyser.para.fit$coef[2]*sort(eruptions))
mtext("(a)")

plot(eruptions, geyser.para.fit$resi,xlab="duration (min)",
     ylab="residuals (min)",type="n")
points(eruptions, geyser.para.fit$resi, pch=".",cex=1)
mtext("(b)")
dev.off()

# fit cubic spline to residuals and original data
postscript(file="geyser2.ps",height=2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(eruptions, geyser.para.fit$resi,xlab="duration (min)",
     ylab="residuals (min)",type="n")
points(eruptions, geyser.para.fit$resi, pch=".",cex=1)
lines(smooth.spline(eruptions,geyser.para.fit$resi))
mtext("(a)")

plot(eruptions,waiting,xlab="duration (min)",ylab="waiting (min)",type="n")
points(eruptions,waiting,pch=".",cex=1)
lines(smooth.spline(eruptions,waiting,all.knots=T))
mtext("(b)")

dev.off()

# detect change point
x <- (eruptions-min(eruptions))/diff(range(eruptions))
aic <- gcv <- NULL
for (t in seq(.25,.55,by=.001)) {
  fit <- ssr(waiting~x+(x>t),rk=cubic(x))
  aic <- c(aic, length(x)*log(sum(fit$resi**2)) + 2*fit$df)
  gcv <- c(gcv, fit$rkpk.obj$score)
}

postscript(file="geyser3.ps",height=1.8,width=4.4,horizontal=F,pointsize=10)
par(mfrow=c(1,3),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)
t <- .397 # midpoint of gcv
z <- 1*(x>t)
geyser.ps.fit <- ssr(waiting~x+z,rk=cubic(x))
grid <- seq(0,1,len=200)
geyser.ps.pred <- predict(geyser.ps.fit, 
    newdata=data.frame(x=grid, z=1*(grid>t)),pstd=T)
grid1 <- grid*diff(range(eruptions))+min(eruptions)
plot(eruptions, waiting,xlab="duration (min)",
     ylab="waiting (min)",type="n", ylim=c(40,100))
polygon(c(grid1,rev(grid1)),
        c(geyser.ps.pred$fit-1.96*geyser.ps.pred$pstd,
          rev(geyser.ps.pred$fit+1.96*geyser.ps.pred$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(eruptions,waiting,pch=".",cex=1)
lines(grid1, geyser.ps.pred$fit)
#axis(3, at=seq(0,1,by=.2)*diff(range(eruptions))+min(eruptions), 
#  labels=as.character(seq(0,1,by=.2)), tck=.02, padj=4, cex.axis=.7)
mtext("(a)")

plot(eruptions, geyser.ps.fit$resi, xlab="duration (min)",
     ylab="residuals (min)",type="n")
points(eruptions, geyser.ps.fit$resi, pch=".",cex=1)
lines(smooth.spline(eruptions,geyser.ps.fit$resi))
mtext("(b)")

saic <- (aic-min(aic))/(max(aic)-min(aic))
sgcv <- (gcv-min(gcv))/(max(gcv)-min(gcv))
plot(seq(.25,.55,by=.001),sgcv,xlab="t",ylab="scaled AIC/GCV",
     ylim=c(-.1,1.1),type="n")
lines(seq(.25,.55,by=.001),sgcv)
lines(seq(.25,.55,by=.001),saic,lty=2)
mtext("(c)")

dev.off()

# plot smooth components
geyser.cub.fit <- ssr(waiting~x,rk=cubic(x))
geyser.cub.pred <- predict(geyser.cub.fit, terms=c(0,0,1),
    newdata=data.frame(x=grid),pstd=T)
geyser.ps.pred1 <- predict(geyser.ps.fit, terms=c(0,0,0,1),
    newdata=data.frame(x=grid, z=1*(grid>t)),pstd=T)

postscript(file="geyser4.ps",height=2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(eruptions, waiting,xlab="duration (min)", ylim=c(-6,6),
     ylab="smooth component (min)",type="n")
polygon(c(grid1,rev(grid1)),
        c(geyser.cub.pred$fit-1.96*geyser.cub.pred$pstd,
          rev(geyser.cub.pred$fit+1.96*geyser.cub.pred$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(grid1, geyser.cub.pred$fit)
abline(0,0,lty=2)
mtext("(a)")

plot(eruptions, waiting,xlab="duration (min)", ylim=c(-.05,.05),
     ylab="smooth component (min)",type="n")
polygon(c(grid1,rev(grid1)),
        c(geyser.ps.pred1$fit-1.96*geyser.ps.pred1$pstd,
          rev(geyser.ps.pred1$fit+1.96*geyser.ps.pred1$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(grid1, geyser.ps.pred1$fit)
abline(0,0,lty=2)
mtext("(b)")

dev.off()

# test
anova(geyser.cub.fit, simu.size=500)
anova(geyser.ps.fit, simu.size=500)

geyser.ps.fit.m <- ssr(waiting~x+z,rk=cubic(x),spar="m")
anova(geyser.ps.fit.m, simu.size=500)
