#Code for motorcycle data

library(MASS)
library(assist)
attach(mcycle)

postscript(file="mcycle1.ps",height=2,width=3.5,horizontal=F,pointsize=8)
par(mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)
plot(times, accel, xlab="time (ms)", ylab="acceleration (g)",
     xlim=c(0,60), type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
points(times, accel, pch="o", cex=.5)
tmp <- lm(accel~times+I(times^2)+I(times^3)+I(times^4)
      +I(times^5)+I(times^6)+I(times^7)+I(times^8)+I(times)^9
      +I(times^10)+I(times^11)+I(times^12)+I(times^13)+I(times^14)
      +I(times^15)+I(times^16)+I(times^17)+I(times^18)+I(times^19)
      +I(times^20))
mcycle.para.fit <- step(tmp, direction="both", trace=0)
lines(seq(min(times),max(times),len=100), 
      predict(mcycle.para.fit,
        data.frame(times=seq(min(times),max(times),len=100))))
dev.off()

postscript(file="mcycle2.ps",height=2,width=3.5,horizontal=F,pointsize=8)
par(mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)
plot(times, accel, xlab="time (ms)", ylab="acceleration (g)",
     xlim=c(0,60), type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
  label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
points(times, accel, pch="o", cex=.5)
lines(smooth.spline(times, accel, all.knots=TRUE))
dev.off()

mcycle.linear.fit <- ssr(accel~1,rk=linear2(times))
mcycle.cubic.fit <- ssr(accel~times,rk=cubic2(times))
T <- cbind(1,times)
Sigma <- cubic2(times)
mcycle.cubic.fit1 <- ssr(accel~T-1,rk=Sigma)
mcycle.cubic.fit2 <- ssr(accel~times, rk=cubic(times), scale=TRUE)
summary(mcycle.cubic.fit)
summary(mcycle.cubic.fit1)
summary(mcycle.cubic.fit2)

mcycle.quintic.fit <- ssr(accel~times+I(times^2),rk=quintic2(times))
mcycle.septic.fit <- ssr(accel~times+I(times^2)+I(times^3),
    rk=septic2(times))

grid <- seq(min(times),max(times),len=50)
mcycle.linear.pred <- predict(mcycle.linear.fit,
    data.frame(times=grid))
mcycle.cubic.pred <- predict(mcycle.cubic.fit,
    data.frame(times=grid))

postscript(file="mcycle3.ps",height=2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,0)+.1) 

plot(times, accel, xlab="time (ms)", ylab="acceleration (g)",
     xlim=c(0,60), type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
    label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
polygon(c(grid,rev(grid)), 
        c(mcycle.linear.pred$fit-1.96*mcycle.linear.pred$pstd,
          rev(mcycle.linear.pred$fit+1.96*mcycle.linear.pred$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(times, accel, pch="o", cex=.5)
lines(grid,mcycle.linear.pred$fit)
mtext("Linear spline fit")

par(mgp=c(2,1,0), mar=c(3,2,1,1))
plot(times, accel, xlab="time (ms)", ylab="",
     xlim=c(0,60), type="n", axes=F)
box()
axis(1, at=c(0,10,20,30,40,50,60), 
    label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
polygon(c(grid,rev(grid)), 
        c(mcycle.cubic.pred$fit-1.96*mcycle.cubic.pred$pstd,
          rev(mcycle.cubic.pred$fit+1.96*mcycle.cubic.pred$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(times, accel, pch="o", cex=.5)
lines(grid,mcycle.cubic.pred$fit)
mtext("Cubic spline fit")

dev.off()

# scale times first
x <- (times-min(times))/(max(times)-min(times)) 
mcycle.cubic.fit3 <- ssr(accel~x, rk=cubic(x))  

# fit with UBR and Rice's estimate of variance
vrice <- mean((diff(accel))**2)/2
mcycle.ubr.1 <- ssr(accel~x,rk=cubic(x),spar="u",varht=vrice)
summary(mcycle.ubr.1)

# fit with UBR and Tong and Wang's estimate of variance 
d <- s <- NULL
for (i in 1:132) {
for (j in (i+1):133) {
  d <- c(d, (x[i]-x[j])**2)
  s <- c(s, (accel[i]-accel[j])**2/2)
}}
# zero bandwidth such that only replicates are picked
vtw <- coef(lm(s~d,subset=d==0))[1]
mcycle.ubr.2 <- ssr(accel~x,rk=cubic(x),spar="u",varht=vtw)
summary(mcycle.ubr.2)

# fit with GCV
mcycle.gcv <- ssr(accel~x,rk=cubic(x),spar="v")
summary(mcycle.gcv)

# fit with GML
mcycle.gml <- ssr(accel~x,rk=cubic(x),spar="m")
summary(mcycle.gml)

# partial spline for change-points
# detect change point in [.2,.25]
aic <- gcv <- NULL
for (t in seq(.2,.25,by=.002)) {
  tmp <- ssr(accel~x+I((x-t)*(x>t)),rk=cubic(x))
  aic <- c(aic, length(x)*log(sum(tmp$resi**2)) + 2*tmp$df)
  gcv <- c(gcv, tmp$rkpk.obj$score)
}

postscript(file="mcycle4.ps",height=2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

t <- .214
s <- (x-t)*(x>t)
mcycle.ps.fit1 <- ssr(accel~x+s,rk=cubic(x))
grid <- seq(0,1,len=100)
mcycle.ps.pred1 <- predict(mcycle.ps.fit1, 
  newdata=data.frame(x=grid, s=(grid-t)*(grid>t)))
grid1 <- grid*(max(times)-min(times))+min(times)
plot(times, accel, xlab="time (ms)", ylab="acceleration (g)",
     xlim=c(0,60), type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
  label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
polygon(c(grid1,rev(grid1)),
        c(mcycle.ps.pred1$fit-1.96*mcycle.ps.pred1$pstd,
          rev(mcycle.ps.pred1$fit+1.96*mcycle.ps.pred1$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(times, accel, pch="o", cex=.5)
lines(grid1, mcycle.ps.pred1$fit)
mtext("(a)")

saic <- (aic-min(aic))/(max(aic)-min(aic))
sgcv <- (gcv-min(gcv))/(max(gcv)-min(gcv))
plot(seq(.2,.25,by=.002),gcv,xlab="t",ylab="scaled AIC/GCV",
     ylim=c(-.1,1.1),type="n")
lines(seq(.2,.25,by=.002),sgcv)
lines(seq(.2,.25,by=.002),saic,lty=2)
mtext("(b)")

dev.off()

# detect potential change points in [.05,.95]
tgrid <- seq(0.05,.95,len=200)
tstat <- NULL
for (t in tgrid) {
  s <- (x-t)*(x>t)
  tmp <- ssr(accel~x+s,rk=cubic(x))
  tmppred <- predict(tmp,terms=c(0,0,1,0),
    newdata=data.frame(x=.5,s=1))
  tstat <- c(tstat,tmppred$fit/tmppred$pstd)
}

t1 <- .2128
t2 <- .3666
t3 <- .5113
s1 <- (x-t1)*(x>t1)
s2 <- (x-t2)*(x>t2)
s3 <- (x-t3)*(x>t3)
mcycle.ps.fit2 <- ssr(accel~x+s1+s2+s3,rk=cubic(x))
grid <- seq(0,1,len=100)
mcycle.ps.pred2 <- predict(mcycle.ps.fit2, 
  newdata=data.frame(x=grid,
                     s1=(grid-t1)*(grid>t1), 
                     s2=(grid-t2)*(grid>t2), 
                     s3=(grid-t3)*(grid>t3)))
grid1 <- grid*(max(times)-min(times))+min(times)

postscript(file="mcycle5.ps",height=3.6,width=4.4,horizontal=F,pointsize=10)
layout(matrix(c(1,0,2)),heights=c(2,lcm(.1),1))
par(mgp=c(2,1,0), mar=c(3,3,3,1)+.1)

plot(times, accel, xlab="", ylab="acceleration (g)",
     xlim=c(0,60), ylim=c(-140,80),type="n", axes=F)
box()
axis(2)
axis(3, at=seq(0,1,by=.1)*diff(range(times))+min(times),
  labels=as.character(seq(0,1,by=.1)), cex=.5)
mtext("x",3,2)
polygon(c(grid1,rev(grid1)),
        c(mcycle.ps.pred2$fit-1.96*mcycle.ps.pred2$pstd,
          rev(mcycle.ps.pred2$fit+1.96*mcycle.ps.pred2$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(times, accel, pch="o", cex=.5)
lines(grid1, mcycle.ps.pred2$fit)

plot(times, accel, xlab="time (ms)", ylab="t-statistics", 
    ylim=range(tstat), xlim=c(0,60), type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
    label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
lines(tgrid*(max(times)-min(times))+min(times), tstat, type="h")

dev.off()

# Estimate variance function and recompute CI's
r <- residuals(mcycle.ps.fit2)
y <- log(r**2)
mcycle.v <- ssr(y~x, cubic(x), spar="m")
summary(mcycle.v)

mcycle.ps.fit3 <- update(mcycle.ps.fit2, weights=exp(mcycle.v$fit))
mcycle.ps.pred3 <- predict(mcycle.ps.fit3, 
  newdata=data.frame(x=grid,
                     s1=(grid-t1)*(grid>t1), 
                     s2=(grid-t2)*(grid>t2), 
                     s3=(grid-t3)*(grid>t3)))
grid1 <- grid*(max(times)-min(times))+min(times)

postscript(file="mcycle6.ps",height=2,width=4.4,
  horizontal=F,pointsize=8)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(times, y, xlab="time (ms)", ylab="log(squared residual)",
     xlim=c(0,60), type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
    label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
points(times, y, pch="o", cex=.5)
lines(times, mcycle.v$fit)
mtext("(a)")

plot(times, accel, xlab="time (ms)", ylab="acceleration (g)", 
     xlim=c(0,60), ylim=c(-150,80), type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
    label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
polygon(c(grid1,rev(grid1)),
        c(mcycle.ps.pred3$fit-1.96*mcycle.ps.pred3$pstd,
          rev(mcycle.ps.pred3$fit+1.96*mcycle.ps.pred3$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(times, accel, pch="o", cex=.5)
lines(grid1, mcycle.ps.pred3$fit)
mtext("(b)")

dev.off()

# Estimate variance using the two-step procedure
# step 1
t1 <- .2128
t2 <- .3666
t3 <- .5113
s1 <- (x-t1)*(x>t1)
s2 <- (x-t2)*(x>t2)
s3 <- (x-t3)*(x>t3)
mcycle.ps.fit4 <- ssr(accel~x+s1+s2+s3,rk=cubic(x))

# step 2
z1 <- residuals(mcycle.ps.fit4)**2
z1[z1<.00001] <- .00001

mcycle.v.1 <- ssr(z1~x, cubic(x), limnla=c(-6,-1),
 family="gamma", spar="u", varht=1)

# step 3
mcycle.ps.fit5 <- update(mcycle.ps.fit4, weights=mcycle.v.1$fit)


# Estimate variance using the differences
# step 1
z2 <- diff(accel)**2/2
z2[z2<.00001] <- .00001

# step 2
n <- length(x)
xx <- (x[1:(n-1)]+x[2:n])/2
mcycle.v.2 <- ssr(z2~xx, cubic(xx), limnla=c(-6,-1),
 family="gamma", spar="u", varht=1)

# step 3
w <- predict(mcycle.v.2, newdata=data.frame(xx=x), pstd=F)
mcycle.ps.fit6 <- update(mcycle.ps.fit4, weights=exp(w$fit))

# joint estimation of mean and variance functions
jemv <- function(x, y, prec=1e-6, maxit=30) {
  t1 <- .2128; t2 <- .3666; t3 <- .5113  
  s1 <- (x-t1)*(x>t1)
  s2 <- (x-t2)*(x>t2)
  s3 <- (x-t3)*(x>t3)
  err <- 1e20; eta <- rep(1,length(x))
  while (err>prec&maxit>0) {
    fitf <- ssr(y~x+s1+s2+s3,cubic(x),weights=exp(eta))
    z <- fitf$resi**2
    z[z<.00001] <- .00001
    fitv <- ssr(z~x, cubic(x), limnla=c(-5,-1),
     family="gamma", spar="u", varht=1)
    oldeta <- eta
    eta <- fitv$rkpk$eta
    err <- sqrt(mean(((eta-oldeta)/(1+abs(oldeta)))**2))
    maxit <- maxit-1
print(c(err,maxit))
  }
  return(list(fitf=fitf,fitv=fitv))
}
mcycle.mv <- jemv(x, accel)


grid <- seq(0,1,len=100)
grid1 <- grid*(max(times)-min(times))+min(times)
mcycle.v.pred1 <- predict(mcycle.v.1, newdata=data.frame(x=grid))
mcycle.v.pred2 <- predict(mcycle.v.2, newdata=data.frame(xx=grid))
tt <- (times[1:(n-1)]+times[2:n])/2
mcycle.v.pred3 <- predict(mcycle.mv$fitv, newdata=data.frame(x=grid))
z3 <- residuals(mcycle.mv$fitf)**2
z3[z3<.00001] <- .00001

yr <- log(range(z1,z2,z3))
postscript(file="mcycle7.ps",height=1.8,width=4.4,
  horizontal=F,pointsize=10)
par(mfrow=c(1,3), mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(times, log(z1), xlab="time (ms)", ylab="log(squared residual)",
     xlim=c(0,60), ylim=yr, type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
    label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
polygon(c(grid1,rev(grid1)),
        c(mcycle.v.pred1$fit-1.96*mcycle.v.pred1$pstd,
          rev(mcycle.v.pred1$fit+1.96*mcycle.v.pred1$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(times, log(z1), pch="o", cex=.5)
lines(grid1, mcycle.v.pred1$fit)
mtext("(a)")

plot(tt, log(z2), xlab="time (ms)", ylab="log(squared difference)",
     xlim=c(0,60), ylim=yr, type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
    label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
polygon(c(grid1,rev(grid1)),
        c(mcycle.v.pred2$fit-1.96*mcycle.v.pred2$pstd,
          rev(mcycle.v.pred2$fit+1.96*mcycle.v.pred2$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(tt, log(z2), pch="o", cex=.5)
lines(grid1, mcycle.v.pred2$fit)
mtext("(b)")

plot(times, log(z3), xlab="time (ms)", ylab="log(squared residual)",
     xlim=c(0,60), ylim=yr, type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
    label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
polygon(c(grid1,rev(grid1)),
        c(mcycle.v.pred3$fit-1.96*mcycle.v.pred3$pstd,
          rev(mcycle.v.pred3$fit+1.96*mcycle.v.pred3$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(times, log(z3), pch="o", cex=.5)
lines(grid1, mcycle.v.pred3$fit)
mtext("(c)")

dev.off()


# Estimate variance using the two-step procedure with DGML
DGML <- function(th, nlaht, y, S, Q) {
 if (length(th)==1) {
  nlaht <- th
  Qt <- Q
 }
 if (length(th)>1) {
  theta <- 10^th
  Qt <- 0
  for (i in 1:dim(Q)[3]) Qt <- Qt + theta[i]*Q[,,i]
 }
 fit <- try(ssr(y~S-1, Qt, limnla=nlaht, family="gamma",
                spar="u", varht=1))
 if(class(fit)=="ssr") {
  fht <- fit$rkpk$eta
  tmp <- y*exp(-fht)
  uc <- 1-tmp
  yt <- fht-uc
  qr.result <- qr(S)
  qrq <- qr.Q(qr.result, complete=T)
  q2 <- qrq[, (ncol(S)+1):nrow(S)]
  V <- t(q2)%*%Qt%*%q2
  l <- eigen((V + t(V))/2)
  U <- l$vectors
  e <- l$values
  z <- t(U)%*%t(q2)%*%yt
  delta <- 10^{-nlaht}
  GML <- sum(tmp+fht)-sum(uc^2)/2+sum(log(delta*e+1))/2+sum(z^2/(delta*e+1))/2
  return(GML)
 }
 else return(1e10)
}


# step 2
S <- cbind(1,x); Q <- cubic(x)
lgrid <- seq(-6,-2,by=.1)
gml <- sapply(lgrid, DGML, nlaht=0, y=z1, S=S, Q=Q)
nlaht <- lgrid[order(gml)[1]]
mcycle.v.g4 <- ssr(z1~x, cubic(x), limnla=nlaht,
 family="gamma", spar="u", varht=1)

# step 3
mcycle.ps.fit7 <- update(mcycle.ps.fit4, weights=mcycle.v.g4$fit)

grid <- seq(0,1,len=100)
grid1 <- grid*(max(times)-min(times))+min(times)
mcycle.ps.pred5 <- predict(mcycle.ps.fit7,
  newdata=data.frame(x=grid,
                     s1=(grid-t1)*(grid>t1),
                     s2=(grid-t2)*(grid>t2),
                     s3=(grid-t3)*(grid>t3)))
mcycle.v.pred4 <- predict(mcycle.v.g4, newdata=data.frame(x=grid))
postscript(file="mcycle8.ps",height=1.8,width=4.4,
  horizontal=F,pointsize=10)
par(mfrow=c(1,3), mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(lgrid, gml, xlab=expression(paste(log,scriptscriptstyle(10),
     "(n",lambda,")")),ylab="DGML", type="l")
segments(nlaht,840,nlaht,842)
mtext("(a)")

plot(times, log(z1), xlab="time (ms)", ylab="log(squared residual)",
     xlim=c(0,60), type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
    label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
polygon(c(grid1,rev(grid1)),
        c(mcycle.v.pred4$fit-1.96*mcycle.v.pred4$pstd,
          rev(mcycle.v.pred4$fit+1.96*mcycle.v.pred4$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(times, log(z1), pch="o", cex=.5)
lines(grid1, mcycle.v.pred4$fit)
mtext("(b)")

plot(times, accel, xlab="time (ms)", ylab="acceleration (g)", 
     xlim=c(0,60), ylim=c(-150,80), type="n", axes=F)
box()
axis(2)
axis(1, at=c(0,10,20,30,40,50,60), 
    label=as.character(c(0,10,20,30,40,50,60)), cex=.5)
polygon(c(grid1,rev(grid1)),
        c(mcycle.ps.pred5$fit-1.96*mcycle.ps.pred5$pstd,
          rev(mcycle.ps.pred5$fit+1.96*mcycle.ps.pred5$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(times, accel, pch="o", cex=.5)
lines(grid1, mcycle.ps.pred5$fit)
mtext("(c)")

dev.off()







