# Code for melanoma data

library(assist)
library(fda)
attach(melanoma)
x <- year-1936
y <- incidence

tau <- .58
tx <- tau*x

# fit a linear-periodic spline
mela.linper <- ssr(y~tx+cos(tx)+sin(tx),
                   rk=lspline(tx,type="linSinCos"),spar="m")

# fit a cubic spline
mela.cub <- ssr(y~x,rk=cubic(x),scale=T,spar="m")

postscript(file="melanoma1.ps",height=2.3,width=3,horizontal=F,pointsize=8)
par(mgp=c(2,1,0),mar=c(3,3,1,1)+.1)
plot(x+1936,y,ylab="cases per 100,000",xlab="year",type="n",axes=F,cex=.5)
box()
axis(1,at=c(1940,1950,1960,1970),cex=.5,
  labels=as.character(c(1940,1950,1960,1970)))
axis(2,cex=.5)
points(x+1936,y,pch="o",cex=.5)
lines(x+1936,mela.linper$fit,lty=1)
lines(x+1936,mela.cub$fit,lty=2)

dev.off()
