# Code for monotone spline simulation

library(assist)

set.seed(2374)
n <- 50
x <- seq(0,1,len=n)
f <- 1-exp(-6*x)
s <- .1
nsim <- 100
cubfit <- incfit <- NULL
totsim <- 0
while (totsim<nsim) {
 y <- f+s*rnorm(n)
 tmp <- try(inc(y,x,spar="m"),T)
 if (class(tmp)!="try-error") {
  incfit <- cbind(incfit,tmp$y.fit)
  cubfit <- cbind(cubfit,ssr(y~I(x-.5),cubic(x),spar="m")$fit)
  totsim <- totsim+1
 }
}
cubmse <- apply((cubfit-f)**2,2,mean)
incmse <- apply((incfit-f)**2,2,mean)

postscript(file="inc.ps",height=2.2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

boxplot(log(cubmse),log(incmse),ylab="log(MSE)",#ylim=c(-12,-3),
 names=c("cubic","monotone"))
mtext("(a)")

plot(x, y, type="n")
mtext("(b)")
points(x,y,pch="o",cex=.5)
lines(x, f, lwd=2)
lines(x, cubfit[,1], lty=2)
lines(x, incfit[,1], lty=1)
legend(.4, .4, lty=c(1,2,1), lwd=c(2,1,1),
       legend=c("true function", "cubic spline",
                "monotone"), cex=.7)

dev.off()


