# Simulation code for MSE and smoothing parameter selection

set.seed(3857)
n <- 73
K <- (n-1)/2
s <- .5
x <- 1:n/n
f <- sin(4*pi*x**2)
a <- mean(f)
A <- 1
for (i in 1:K) {
  a <- c(a,sqrt(2)*mean(f*sin(2*pi*i*x)),sqrt(2)*mean(f*cos(2*pi*i*x)))
  A <- cbind(A,sqrt(2)*sin(2*pi*i*x),sqrt(2)*cos(2*pi*i*x))
}
g <- as.vector(A%*%a)
y <- g+s*rnorm(n)

postscript(file="simumse.ps",height=2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)
plot(x,y,type="n")
points(x,y,cex=.5)
lines(x,g)
mtext("(a)")

nlambda <- seq(-8,-2,len=100)
b <- a[seq(2,n,by=2)]**2+a[seq(3,n,by=2)]**2
bias <- var <- rep(-1,100)
for (i in 1:100) {
  tmp <- 10**(nlambda[i])/n  
  bias[i] <- sum((tmp/((2*pi*(1:K))**(-4)+tmp))**2*b)
  var[i] <- s**2*(1+2*sum(1/(1+(2*pi*(1:K))**4*tmp)**2))/n
}
mse <- bias+var

plot(nlambda,bias,xlab=expression(paste(log,scriptscriptstyle(10),
     "(n",lambda,")")),ylab="squared bias,variance, and MSE",
type="n",ylim=c(0,.3))
lines(nlambda,bias,lty=2)
lines(nlambda,var,lty=3)
lines(nlambda,mse,lty=1)
mtext("(b)")
dev.off()

library(assist)
vrice <- function(y) {
  n <- length(y)
  sum((y[1:(n-1)]-y[2:n])**2)/2/(n-1)
}

nrep <- 50
pse <- mse+s^2
postscript(file="simuubr.ps",height=2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,1,0)+0.1)

plot(nlambda,pse,xlab=expression(paste(log,scriptscriptstyle(10),
     "(n",lambda,")")),ylab="PSE and UBR",
     type="n",ylim=c(0.1,.6))
set.seed(3857)
for (i in 1:nrep) {
  y <- g+s*rnorm(n)
  tmp <- ssr(y~1,periodic(x),spar="u",varht=s^2,limnla=c(-8,-2),
            control=list(job=99))
  ubr <- tmp$rkpk.obj$score
  lines(nlambda,ubr,lty=2,lwd=.8,col=gray(0:8/8)[7])
  segments(tmp$rkpk.obj$nlaht,.1,tmp$rkpk.obj$nlaht,.125,
           lty=2,lwd=.8,col=gray(0:8/8)[7])
}
lines(nlambda,pse,lty=1)
segments(nlambda[pse==min(pse)],.1,nlambda[pse==min(pse)],.15)
mtext("UBR with true variance")

par(mgp=c(2,1,0),mar=c(3,2,1,1))
plot(nlambda,pse,xlab=expression(paste(log,scriptscriptstyle(10),
     "(n",lambda,")")),ylab="", type="n",ylim=c(0.1,.6),axes=F)
box()
axis(1)
set.seed(3857)
for (i in 1:nrep) {
  y <- g+s*rnorm(n)
  vr <- vrice(y)
  tmp <- ssr(y~1,periodic(x),spar="u",varht=vr,limnla=c(-8,-2),
            control=list(job=99))
  ubr <- tmp$rkpk.obj$score
  lines(nlambda,ubr,lty=2,lwd=.8,col=gray(0:8/8)[7])
  segments(tmp$rkpk.obj$nlaht,.1,tmp$rkpk.obj$nlaht,.125,
           lty=2,lwd=.8,col=gray(0:8/8)[7])
}
lines(nlambda,pse,lty=1)
segments(nlambda[pse==min(pse)],.1,nlambda[pse==min(pse)],.15)
mtext("UBR with Rice estimator")

dev.off()

A <- array(NA,c(n,n,length(nlambda)))
for (i in 1:length(nlambda))
  A[,,i] <- hat.ssr(ssr(y~1,periodic(x),
                   limnla=c(nlambda[i],nlambda[i])))

postscript(file="simucv.ps",height=2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)

plot(nlambda,pse,xlab=expression(paste(log,scriptscriptstyle(10),
     "(n",lambda,")")),ylab="PSE and CV",
     type="n",ylim=c(0.1,.6))

set.seed(3857)
for (j in 1:nrep) {
  y <- g+s*rnorm(n)
  cv <- NULL
  for (i in 1:length(nlambda)) 
    cv <- c(cv,mean((A[,,i]%*%y-y)**2/(1-diag(A[,,i]))**2))
  lines(nlambda,cv,lty=2,lwd=.8,col=gray(0:8/8)[7])
  segments(nlambda[cv==min(cv)],.1,nlambda[cv==min(cv)],.125,
           lty=2,lwd=.8,col=gray(0:8/8)[7])
}
lines(nlambda,pse,lty=1)
segments(nlambda[pse==min(pse)],.1,nlambda[pse==min(pse)],.15)
mtext("CV")

plot(nlambda,pse,xlab=expression(paste(log,scriptscriptstyle(10),
     "(n",lambda,")")),ylab="PSE and GCV",
     type="n",ylim=c(0.1,.6))

set.seed(3857)
for (j in 1:nrep) {
  y <- g+s*rnorm(n)
  gcv <- NULL
  for (i in 1:length(nlambda)) 
    gcv <- c(gcv,mean((A[,,i]%*%y-y)**2)/(1-mean(diag(A[,,i])))**2)
  lines(nlambda,gcv,lty=2,lwd=.8,col=gray(0:8/8)[7])
  segments(nlambda[gcv==min(gcv)],.1,nlambda[gcv==min(gcv)],.125,
           lty=2,lwd=.8,col=gray(0:8/8)[7])
}
lines(nlambda,pse,lty=1)
segments(nlambda[pse==min(pse)],.1,nlambda[pse==min(pse)],.15)
mtext("GCV")

dev.off()

