one.bond <- function(price, payment, time, name, spar="m", limnla=c(-3,6)) {
 # pre-processing the data
 # the data has to be sorted by the name
 group <- as.vector(table(name))
 y <- price[cumsum(group)]
 n.time <- length(time)

 # create points for Gaussian quadrature, 3 points in each interval 
 s.time <- sort(time)
 x1.y <- c(0, s.time[-n.time])
 x2.y <- s.time-x1.y
 org.ord <- match(1:n.time, (1:n.time)[order(time)])
 q.time <- as.vector(rep(1,3)%o%x1.y+c(0.1127017,0.5,0.8872983)%o%x2.y)

 # initial values for f
 f0 <- function(x) rep(0.04, length(x))    
 f.old <- f.val <- f0(q.time)

 # create s and q matrices
 S <- cbind(time,time*time/2.0)               
 Lambda <- int1(s.time,rep(1,3*length(s.time)))[org.ord,org.ord]
 Lint <- int.f(s.time,q.time,rep(1,3*length(s.time)))[org.ord,]

 # begin iteration
 iter <- cover <- 1    
 repeat {
  fint <- int.s(f.val,s.time)[org.ord]
  X <- diagComp(matrix(payment*exp(-fint),nrow=1),group)
  ytilde <- X%*%(1+fint)-y
  T <- X%*%S; Q <- X%*%Lambda%*%t(X)
  fit <- ssr(ytilde~T-1, Q, spar=spar, limnla=limnla)
  dd <- fit$coef$d; cc <- fit$coef$c
  f.val <- as.vector((cc%*%X)%*%Lint+dd[1]+dd[2]*q.time)        
  cover <- mean((f.val-f.old)^2)
  if(cover<1.e-6 || iter>20) break
  iter<- iter+1; f.old <- f.val
 }
 tmp <- -int.s(f.val,s.time)[org.ord]
 yhat <- apply(diagComp(matrix(payment*exp(tmp),nrow=1),group),1,sum)
 sigma <- sqrt(sum((y-yhat)^2)/(length(y)-fit$df))
 list(fit=fit, iter=c(iter, cover), call=match.call(), 
      f.val=f.val, q.time=q.time, dc=exp(tmp),
      y=list(y=y,yhat=yhat), sigma=sigma)
}    

