# Code for child growth data

library(assist)
library(fda)
attach(onechild)

x <- day/365
y <- height
grid <- seq(0,max(x),len=100)
onechild.inc.fit <- inc(y, x, spar="m", grid=grid)
onechild.cub.fit <- ssr(y~x, cubic(x), spar="m")

## compute bootstrap confidence intervals
nboot <- 10000
set.seed <- 23057
yhat <- onechild.inc.fit$y.fit
resi <- y-yhat
fb <- gb <- NULL
totboot <- 0
while (totboot<nboot) {
 yb <- yhat+sample(resi, replace=T)
 tmp <- try(inc(yb, x, spar="m", grid=grid))
 if(class(tmp)!="try-error"&tmp$iter[2]<1.e-6) {
  fb <- cbind(fb, (tmp$pred$f-onechild.inc.fit$pred$f)/tmp$sigma)
  gb <- cbind(gb, (tmp$pred$y-onechild.inc.fit$pred$y)/tmp$sigma)
  totboot <- totboot+1
 }
}

gl <- onechild.inc.fit$pred$y-
  onechild.inc.fit$sigma*apply(gb,1,quantile,prob=.975)
gu <- onechild.inc.fit$pred$y-
  onechild.inc.fit$sigma*apply(gb,1,quantile,prob=.025)

hl <- exp(onechild.inc.fit$pred$f-
  onechild.inc.fit$sigma*apply(fb,1,quantile,prob=.975))/365
hu <- exp(onechild.inc.fit$pred$f-
  onechild.inc.fit$sigma*apply(fb,1,quantile,prob=.025))/365

postscript(file="onechild.ps",height=2.2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(day,y,xlab="day",ylab="height (cm)",type="n")
mtext("(a)", cex=1.1)
polygon(c(grid*365,rev(grid*365)),c(gl,rev(gu)),
 col=gray(0:8/8)[8], border=NA)
points(day,y,pch="o",cex=.5)
lines(grid*365, predict(onechild.cub.fit,data.frame(x=grid))$fit, lty=2)
lines(grid*365, onechild.inc.fit$pred$y)

h <- exp(onechild.inc.fit$pred$f)
plot(grid*365,h/365, type="n", ylim=range(hl,hu),
 xlab="day", ylab="growth velocity (cm/day)")
mtext("(b)", cex=1.1)
polygon(c(grid*365,rev(grid*365)),c(hl,rev(hu)),
 col=gray(0:8/8)[8], border=NA)
lines(grid*365,h/365)

dev.off()


