# Code for paramecium caudatum data 

library(assist)
data(paramecium)
attach(paramecium)

x <- c(0,2:25)
y <- c(2,10,10,11,21,56,104,137,165,194,217,199,201,182,192,179,
       190,206,209,196,195,234,210,210,180)

para.nls <- nls(density~b1/(1 + b2*exp(-b3*day)),
               start=list(b1=202,b2=164,b3=0.74))

# RK for logistic spline
logit.rk <- function(x,a,d,r) {
  tmp1 <- x%o%rep(1,length(x)) 
  tmp2 <- (tmp1+t(tmp1)-abs(tmp1-t(tmp1)))/2
  tmp3 <- exp(-r*a)-exp(-r*tmp2)
  tmp4 <- exp(-2*r*a)-exp(-2*r*tmp2)
  tmp5 <- 1/((1+d*exp(-r*x))%o%(1+d*exp(-r*x)))
  (tmp2-a+2*d*tmp3/r+d**2*tmp4/(2*r))*tmp5
}
bh <- coef(para.nls)
para.logit <- ssr(density~I(1/(1+bh[2]*exp(-bh[3]*day)))-1,
                  rk=logit.rk(day,0,bh[2],bh[3]),spar="m")
para.cub <- ssr(density~day,rk=cubic(day),scale=T,spar="m")

para.p.nls <- predict(para.nls)
para.p.logit <- predict(para.logit)
para.p.cub <- predict(para.cub)
d.logit <- para.p.logit$fit-para.p.nls
d.cub <- para.p.cub$fit-para.p.nls

postscript(file="paramecium1.ps",height=2.4,width=3,horizontal=F,pointsize=8)
par(mgp=c(2,1,0),mar=c(3,3,1,1)+.1)
  
plot(day,density,ylab="density",xlab="day",type="n",cex=.5)
points(day,density,pch="o",cex=.5)
lines(day,para.p.nls,lty=3)
lines(day,para.p.logit$fit,lty=1)
lines(day,para.p.cub$fit,lty=2)

dev.off()

# estimate r and d
bh <- coef(para.nls)
ls.fun <- function(a) {
  tmp.fit <- ssr(density~I(1/(1+a[1]*exp(-a[2]*day)))-1,
   rk=logit.rk(day,0,a[1],a[2]), spar="m")
  sum((tmp.fit$resi)**2)
}
nlm(ls.fun,c(705,1))

pls.fun <- function(a) {
  tmp.fit <- ssr(density~I(1/(1+a[1]*exp(-a[2]*day)))-1,
   rk=logit.rk(day,0,a[1],a[2]),spar="m")
  sum((tmp.fit$resi)**2)+1000*tmp.fit$rkpk.obj$pen
}
nlm(pls.fun,c(706,.9))$estimate

