# Simulation code for spline smoothing with positive constraint

library(assist)
set.seed(5680)
n <- 50
x <- seq(0,1,len=n)
y <- exp(-6*x)+.1*rnorm(n)

ssrfit <- ssr(y~x, cubic(x))
nnrfit <- nnr(y~exp(f(x)), func=f(u)~list(~u,cubic(u)),
              start=list(f=log(abs(y)+0.001)))
 
nsimu <- 100
grid <- seq(0,1,len=100)
cubfit <- expfit <- NULL
for (i in 1:nsimu) {
  y <- exp(-6*x)+.1*rnorm(n)
  tmp <- predict(ssr(y~x, cubic(x)),newdata=data.frame(x=grid))$fit
  cubfit <- cbind(cubfit,tmp)
  tmp <- intervals(nnr(y~exp(f(x)), func=f(u)~list(~u,cubic(u)),
              start=list(f=log(abs(y)+0.001))),
              newdata=data.frame(x=grid))$f$fit
  expfit <- cbind(expfit, exp(tmp))
}
truef <- exp(-6*grid)
cubmse <- apply((cubfit-truef)**2,2,mean)
expmse <- apply((expfit-truef)**2,2,mean)

postscript(file="positive.ps",height=2.2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

boxplot(log(cubmse),log(expmse),ylab="log(MSE)",ylim=c(-12,-3.5),
 names=c("cubic","exp. trans."))

plot(x, y, type="n")
points(x,y,pch="o",cex=.5)
lines(x, exp(-6*x),lwd=2)
lines(x, ssrfit$fit, lty=2)
lines(x, exp(nnrfit$funcFitted$f), lty=1)
legend(.15, 1, lty=c(1,2,1), lwd=c(2,1,1),
       legend=c("true function", "cubic spline", 
                "exponential transformation"), cex=.7)

dev.off()


