# Code for rock data

library(assist)
attach(rock)

# fit the same PPR model as in Venable and Ripley
y <- log(perm)
x1 <- area/10000; x2 <- peri/10000; x3 <- shape
rock.ppr <- ppr(y~x1+x2+x3, nterms=2, max.terms=5) 

# Spherical Coordinates
b.ppr <- rock.ppr$alpha
a11ini <- acos(b.ppr[3,1]/sqrt(sum(b.ppr[,1]**2)))
a12ini <- atan(b.ppr[2,1]/b.ppr[1,1])
a21ini <- acos(b.ppr[3,2]/sqrt(sum(b.ppr[,2]**2)))
a22ini <- atan(b.ppr[2,2]/b.ppr[1,2])
rock.snr <- snr(y~b0+f1(sin(a11)*cos(a12)*x1+sin(a11)*sin(a12)*x2+cos(a11)*x3)+
                  f2(sin(a21)*cos(a22)*x1+sin(a21)*sin(a22)*x2+cos(a21)*x3),
                func=list(f1(u)~list(~u-1,tp(u)),f2(v)~list(~v-1,tp(v))),
                params=list(b0+a11+a12+a21+a22~1), spar="m",
                start=list(params=c(mean(y),a11ini,a12ini,a21ini,a22ini)),
                control=list(prec.out=1.e-3,maxit.out=50))

a <- rock.snr$coef[-1]
u <- sin(a[1])*cos(a[2])*x1+sin(a[1])*sin(a[2])*x2+cos(a[1])*x3
v <- sin(a[3])*cos(a[4])*x1+sin(a[3])*sin(a[4])*x2+cos(a[3])*x3
ugrid <- seq(min(u),max(u),len=50)
vgrid <- seq(min(v),max(v),len=50)
rock.snr.ci <- intervals(rock.snr,newdata=data.frame(u=ugrid,v=vgrid))

postscript(file="rock.ps",height=2,width=4.4,
  horizontal=F,pointsize=8)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1) 

plot(ugrid,rock.snr.ci$f1$fit,type="n",ylim=c(-4,3),
 ylab=expression(f[1]),xlab="term 1")
polygon(c(ugrid,rev(ugrid)), 
 c(rock.snr.ci$f1$fit-1.96*rock.snr.ci$f1$pstd,
 rev(rock.snr.ci$f1$fit+1.96*rock.snr.ci$f1$pstd)),
 col=gray(0:8/8)[8], border=NA)
lines(ugrid,rock.snr.ci$f1$fit)

plot(vgrid,rock.snr.ci$f2$fit,type="n",ylim=c(-4,3),
 ylab=expression(f[2]),xlab="term 2")
polygon(c(vgrid,rev(vgrid)), 
 c(rock.snr.ci$f2$fit-1.96*rock.snr.ci$f2$pstd,
 rev(rock.snr.ci$f2$fit+1.96*rock.snr.ci$f2$pstd)),
 col=gray(0:8/8)[8], border=NA)
lines(vgrid,rock.snr.ci$f2$fit)

dev.off()


