# Code for seisure data

library(assist)
library(lattice)
data(seizure); attach(seizure)
n <- nrow(seizure)

postscript(file="seizure1.ps",height=4,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(2,1), mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)

plot(t, base, xlab="", ylab="", type="l", axes=F, ylim=c(-300,300), lwd=.5)
box()
axis(2)
axis(1, at=seq(0,60000,by=12000), label=as.character(0:5), cex=.5)
mtext("Baseline", cex=1.2)

plot(t, preseizure, xlab="time (minute)", ylab="", 
     type="l", axes=F, ylim=c(-300,300), lwd=.5)
box()
axis(2)
axis(1, at=seq(0,60000,by=12000), label=as.character(0:5), cex=.5)
mtext("Preseizure", cex=1.2)

dev.off()

# fit as stationary process
# use a subset of periodograms only, excluding frequency 0
x <- seq(1,n-1, by=120)/n
postscript(file="seizure2.ps",height=2.2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)

#y <- pgram(base, x) # results similar
y <- (abs(fft(base))^2/n)[-1][seq(1,n-1, by=120)]
seizure.s.base <- ssr(y~x, periodic(x), spar="u", varht=1, 
 family="gamma",limnla=c(-5,1))
grid <- data.frame(x=seq(0,.5,len=100))
seizure.s.base.p <- predict(seizure.s.base, grid)
plot(200*x[x<.5], log(y[x<.5]), xlab="frequency (HZ)", 
 ylab="log spectrum", type="n", ylim=c(-1,11))
polygon(c(200*grid$x,rev(200*grid$x)), 
 c(seizure.s.base.p$fit-1.96*seizure.s.base.p$pstd,
   rev(seizure.s.base.p$fit+1.96*seizure.s.base.p$pstd)),
   col=gray(0:8/8)[8], border=NA)
points(200*x[x<.5], log(y[x<.5]), pch=".")
lines(200*grid$x, seizure.s.base.p$fit)
mtext("Baseline", cex=1.2)

#y <- pgram(preseizure, x)
y <- (abs(fft(preseizure))^2/n)[-1][seq(1,n-1, by=120)]
seizure.s.pre <- ssr(y~x, periodic(x), spar="u", varht=1, 
 family="gamma",limnla=c(-5,1))
seizure.s.pre.p <- predict(seizure.s.pre, grid)
plot(200*x[x<.5], log(y[x<.5]), xlab="frequency (HZ)", 
 ylab="log spectrum", type="n", ylim=c(-1,11))
polygon(c(200*grid$x,rev(200*grid$x)), 
 c(seizure.s.pre.p$fit-1.96*seizure.s.pre.p$pstd,
   rev(seizure.s.pre.p$fit+1.96*seizure.s.pre.p$pstd)),
 col=gray(0:8/8)[8], border=NA)
points(200*x[x<.5], log(y[x<.5]), pch=".")
lines(200*grid$x, seizure.s.pre.p$fit)
mtext("Preseizure", cex=1.2)

dev.off()


# fit as locally stationary process
pgram <- function(x, freqs) {
 y <- numeric(length(freqs)); tser <- seq(length(x))-1
 for(i in seq(length(freqs))) y[i] <- Mod(sum(x*complex(mod=1,
                                     arg=-2*pi*freqs[i]*tser)))^2
 y/length(x)
}

lpgram <- function(x,times,freqs) {
 nsub <- floor(length(x)/length(times))
 ymat <- matrix(0, length(freqs), length(times))
 for (j in seq(length(times))) {
  ymat[,j] <- pgram(x[((j-1)*nsub+1):(j*nsub)], freqs)
 }
 as.vector(ymat)
}

nf <- nt <- 32
freqs <- 1:nf/(nf+1)
nsub <- floor(n/nt) ## Length of each time segment.
times <- (seq(from=(1+nsub)/2, by=nsub, length=nt))/n

y <- lpgram(base,times,freqs)
ftgrid <- expand.grid(freqs,times)
x1 <- ftgrid[,1] # freq
x2 <- ftgrid[,2] # time
seizure.ls.base <- ssr(y~x2, rk=list(periodic(x1), cubic(x2), 
 rk.prod(periodic(x1),kron(x2-.5)), rk.prod(periodic(x1),cubic(x2))), 
 spar="u", varht=1, family="gamma")
grid <- expand.grid(x1=seq(0,.5,len=20), x2=seq(0,1,len=20))
seizure.ls.base.p <- predict(seizure.ls.base, newdata=grid, pstd=F)

y <- lpgram(preseizure,times,freqs)
seizure.ls.pre <- ssr(y~x2, rk=list(periodic(x1), cubic(x2),
 rk.prod(periodic(x1),kron(x2-.5)), rk.prod(periodic(x1),cubic(x2))),
 spar="u", varht=1, family="gamma")
seizure.ls.pre.p <- predict(seizure.ls.pre, newdata=grid, pstd=F)

postscript(file="seizure3.ps",height=2.5*1.2,width=4.45*1.2,horizontal=F,pointsize=10)
trellis.par.set("axis.line", list(col="transparent"))

xx1 <- 200*grid$x1
xx2 <- 5*grid$x2
p1 <- wireframe(seizure.ls.base.p$fit~xx1*xx2, 
 screen=list(z=-25,x=-75), xlab=list("frequency (HZ)",cex=.7,rot=-8),
 ylab=list("time (minute)",cex=.7,rot=50), 
 zlab=list("log spectrum",cex=.7,rot=90),
 scales=list(arrows=F, cex=.6, col="black", tck=1))
print(p1, split=c(1,1,2,1), more=T)

p2 <- wireframe(seizure.ls.pre.p$fit~xx1*xx2,
 screen=list(z=-25,x=-75), xlab=list("frequency (HZ)",cex=.7,rot=-8),
 ylab=list("time (minute)",cex=.7,rot=50), 
 zlab=list("log spectrum",cex=.7,rot=90),
 scales=list(arrows=F, cex=.6, col="black", tck=1))
print(p2, split = c(2,1,2,1))

dev.off()

# direct GML
DGML <- function(th, nlaht, y, S, Q) {
 if (length(th)==1) {
  nlaht <- th
  Qt <- Q 
 }
 if (length(th)>1) {
  theta <- 10^th
  Qt <- 0
  for (i in 1:dim(Q)[3]) Qt <- Qt + theta[i]*Q[,,i]
 }
 fit <- try(ssr(y~S-1, Qt, limnla=nlaht, family="gamma",
                spar="u", varht=1))
 if(class(fit)=="ssr") {
  fht <- fit$rkpk$eta
  tmp <- y*exp(-fht)
  uc <- 1-tmp
  yt <- fht-uc
  qr.result <- qr(S)
  qrq <- qr.Q(qr.result, complete=T)
  q2 <- qrq[, (ncol(S)+1):nrow(S)]
  V <- t(q2)%*%Qt%*%q2
  l <- eigen((V + t(V))/2)
  U <- l$vectors
  e <- l$values
  z <- t(U)%*%t(q2)%*%yt
  delta <- 10^{-nlaht}
  GML <- sum(tmp+fht)-sum(uc^2)/2+sum(log(delta*e+1))/2+sum(z^2/(delta*e+1))/2
  return(GML)
 }
 else return(1e10)
}


spdest <- function(y, freq, time, process, 
 control=list(optfactr=1e10,limnla=c(-6,1),prec=1e-6,maxit=30)) 
{
 if (process=="S") {
  thhat <- rep(NA, 4)
  S <- as.matrix(rep(1, length(y)))
  Q <- periodic(freq)
  tmp <- try(optim(-2, DGML, y=y, S=S, Q=Q, method="L-BFGS-B",
             control=list(factr=control$optfactr),
             lower=control$limnla[1], upper=control$limnla[2]))
  if (class(tmp)=="try-error") { info <- "optim failure"}
  else {
   if (tmp$convergence==0) {
    nlaht <- tmp$par
    fit <- ssr(y~S-1, Q, limnla=tmp$par, family="gamma", spar="u", varht=1)
    info <- "success" 
   }
   else info <- paste("optim failure", as.character(tmp$convergence))
  }
 }
 if (process=="LS") {
  S <- cbind(1,time-.5)
  Q <- array(NA,c(length(freq),length(freq),4))
  Q[,,1] <- periodic(freq)
  Q[,,2] <- cubic(time)
  Q[,,3] <- rk.prod(periodic(freq), kron(time-.5))
  Q[,,4] <- rk.prod(periodic(freq), cubic(time))

  # compute initial values for optimization of theta
  zz <- log(y)+.57721
  tmp <- try(ssr(zz~S-1, Q, spar="m"),T)
  if (class(tmp)=="ssr") {
   thini <- tmp$rkpk.obj$theta
   nlaht <- tmp$rkpk.obj$nlaht
  }
  else {
   thini <- rep(1,4)
   nlaht <- 1.e-8
  }

  tmp <- try(optim(thini, DGML, nlaht=nlaht,
             y=y, S=S, Q=Q, method="L-BFGS-B", 
             control=list(factr=control$optfactr)))
  if (class(tmp)=="try-error") { info="optim failure" }
  else {
   if (tmp$convergence==0) {
    thhat <- tmp$par
    thetahat <- 10**thhat
    Qt <- 0
    for (i in 1:dim(Q)[3]) Qt <- Qt + thetahat[i]*Q[,,i]
    fit <- ssr(y~S-1, Qt, limnla=nlaht, family="gamma", spar="u", varht=1)
    info <- "success" 
   }
   else { info <- paste("optim failure", as.character(tmp$convergence)) } 
  }
 } 
 return(list(fit=fit, nlaht=nlaht, thhat=thhat, info=info))
}

# fit as stationary process with DGML
x <- seq(1,n-1, by=120)/n
postscript(file="seizure4.ps",height=2.2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)

y <- pgram(base, x)
tmp <- spdest(y, x, 0, "S")
seizure.s.base.dgml <- ssr(y~x, periodic(x), spar="u", varht=1,
 family="gamma",limnla=tmp$nlaht)
grid <- data.frame(x=seq(0,.5,len=100))
seizure.s.base.dgml.p <- predict(seizure.s.base.dgml, grid)
plot(200*x[x<.5], log(y[x<.5]), xlab="frequency (HZ)", 
 ylab="log spectrum", type="n", ylim=c(-1,11))
polygon(c(200*grid$x,rev(200*grid$x)), 
 c(seizure.s.base.dgml.p$fit-1.96*seizure.s.base.dgml.p$pstd,
   rev(seizure.s.base.dgml.p$fit+1.96*seizure.s.base.dgml.p$pstd)),
   col=gray(0:8/8)[8], border=NA)
points(200*x[x<.5], log(y[x<.5]), pch=".")
lines(200*grid$x, seizure.s.base.dgml.p$fit)
mtext("Baseline", cex=1.2)

y <- pgram(preseizure, x)
tmp <- spdest(y, x, x2, "S")
seizure.s.pre.dgml <- ssr(y~x, periodic(x), spar="u", varht=1, 
 family="gamma",limnla=tmp$nlaht)
seizure.s.pre.dgml.p <- predict(seizure.s.pre.dgml, grid)
plot(200*x[x<.5], log(y[x<.5]), xlab="frequency (HZ)", 
 ylab="log spectrum", type="n", ylim=c(-1,11))
polygon(c(200*grid$x,rev(200*grid$x)), 
 c(seizure.s.pre.dgml.p$fit-1.96*seizure.s.pre.dgml.p$pstd,
   rev(seizure.s.pre.dgml.p$fit+1.96*seizure.s.pre.dgml.p$pstd)),
 col=gray(0:8/8)[8], border=NA)
points(200*x[x<.5], log(y[x<.5]), pch=".")
lines(200*grid$x, seizure.s.pre.dgml.p$fit)
mtext("Preseizure", cex=1.2)

dev.off()

# fit as local stationary with DGML
nf <- nt <- 32
freqs <- 1:nf/(nf+1)
nsub <- floor(n/nt) ## Length of each time segment.
times <- (seq(from=(1+nsub)/2, by=nsub, length=nt))/n

ftgrid <- expand.grid(freqs,times)
x1 <- ftgrid[,1] # freq
x2 <- ftgrid[,2] # time
S <- cbind(1,x2-.5)
Q1 <- periodic(x1)
Q2 <- cubic(x2)
Q3 <- rk.prod(periodic(x1), kron(x2-.5))
Q4 <- rk.prod(periodic(x1), cubic(x2))
grid <- expand.grid(x1=seq(0,.5,len=20), x2=seq(0,1,len=20))
Sg <- cbind(1,grid$x2-.5)
Qg1 <- periodic(grid$x1,x1)
Qg2 <- cubic(grid$x2,x2)
Qg3 <- rk.prod(periodic(grid$x1,x1), kron(grid$x2-.5,x2-.5))
Qg4 <- rk.prod(periodic(grid$x1,x1), cubic(grid$x2,x2))

y <- lpgram(base,times,freqs)
tmp <- spdest(y, x1, x2, "LS")
th <- 10**tmp$thhat
Qt <- th[1]*Q1+th[2]*Q2+th[3]*Q3+th[4]*Q4
seizure.ls.base.dgml <- ssr(y~x2, rk=Qt, spar="u", varht=1, 
 family="gamma", limnla=tmp$nlaht)
Qgt <- th[1]*Qg1+th[2]*Qg2+th[3]*Qg3+th[4]*Qg4
seizure.ls.base.dgml.p <- Sg%*%seizure.ls.base.dgml$coef$d+
 Qgt%*%seizure.ls.base.dgml$coef$c

y <- lpgram(preseizure,times,freqs)
tmp <- spdest(y, x1, x2, "LS")
th <- 10**tmp$thhat
Qt <- th[1]*Q1+th[2]*Q2+th[3]*Q3+th[4]*Q4
seizure.ls.pre.dgml <- ssr(y~x2, rk=Qt, spar="u", varht=1,
 family="gamma", limnla=tmp$nlaht)
Qgt <- th[1]*Qg1+th[2]*Qg2+th[3]*Qg3+th[4]*Qg4
seizure.ls.pre.dgml.p <- Sg%*%seizure.ls.pre.dgml$coef$d+
 Qgt%*%seizure.ls.pre.dgml$coef$c

postscript(file="seizure5.ps",height=2.5*1.2,width=4.45*1.2,horizontal=F,pointsize=10)
trellis.par.set("axis.line", list(col="transparent"))

xx1 <- 200*grid$x1
xx2 <- 5*grid$x2
p1 <- wireframe(seizure.ls.base.dgml.p~xx1*xx2, 
 screen=list(z=-25,x=-75), xlab=list("frequency (HZ)",cex=.7,rot=-8),
 ylab=list("time (minute)",cex=.7,rot=50), 
 zlab=list("log spectrum",cex=.7,rot=90),
 scales=list(arrows=F, cex=.6, col="black", tck=1))
print(p1, split=c(1,1,2,1), more=T)

p2 <- wireframe(seizure.ls.pre.dgml.p~xx1*xx2,
 screen=list(z=-25,x=-75), xlab=list("frequency (HZ)",cex=.7,rot=-8),
 ylab=list("time (minute)",cex=.7,rot=50), 
 zlab=list("log spectrum",cex=.7,rot=90),
 scales=list(arrows=F, cex=.6, col="black", tck=1))
print(p2, split = c(2,1,2,1))

dev.off()

# permutation test for baseline, caution: takes a long time
y <- lpgram(base,times,freqs)
full <- spdest(y, x1, x2, "LS")
reduced <- spdest(y, x1, x2, "S")
fitf <- full$fit$rkpk$eta
fitr <- reduced$fit$rkpk$eta
devf <- sum(-1-log(y)+y*exp(-fitf)+fitf)
devr <- sum(-1-log(y)+y*exp(-fitr)+fitr)
cstat <- devr-devf
l2d <- mean((fitf-fitr)**2)

set.seed(4095)
nperm <- 100; totperm <- 0
cstatp <- l2dp <- NULL
while (totperm < nperm) {
print(totperm)
 x2p <- rep(sample(times),rep(nf,nt))
 full <- try(spdest(y, x1, x2p, "LS"))
 if (class(full)!="try-error") {
  totperm <- totperm+1
  fitf <- full$fit$rkpk$eta
  devf <- sum(-1-log(y)+y*exp(-fitf)+fitf)
  cstatp <- c(cstatp, devr-devf)
  l2dp <- c(l2dp, mean((fitf-fitr)**2))
 }
}

print(c(mean(cstatp>cstat),mean(l2dp>l2d)))

# permutation test for preseizure
y <- lpgram(preseizure,times,freqs)
full <- spdest(y, x1, x2, "LS")
reduced <- spdest(y, x1, x2, "S")
fitf <- full$fit$rkpk$eta
fitr <- reduced$fit$rkpk$eta
devf <- sum(-1-log(y)+y*exp(-fitf)+fitf)
devr <- sum(-1-log(y)+y*exp(-fitr)+fitr)
cstat <- devr-devf
l2d <- mean((fitf-fitr)**2)

set.seed(5346)
nperm <- 100; totperm <- 0
cstatp <- l2dp <- NULL
while (totperm < nperm) {
print(totperm)
 x2p <- rep(sample(times),rep(nf,nt))
 full <- try(spdest(y, x1, x2p, "LS"))
 if (class(full)!="try-error") {
  totperm <- totperm+1
  fitf <- full$fit$rkpk$eta
  devf <- sum(-1-log(y)+y*exp(-fitf)+fitf)
  cstatp <- c(cstatp, devr-devf)
  l2dp <- c(l2dp, mean((fitf-fitr)**2))
 }
}

print(c(mean(cstatp>cstat),mean(l2dp>l2d)))


