# Code for star data

library(assist)
data(star); attach(star)

star.fit.1 <- snr(y~(1+b1*x)*f1(log(1+b3*x/b2)/b3),
            func=list(f1(u)~list(~sin(2*pi*u)+cos(2*pi*u),
                                 lspline(u,type="sine1"))),
            params=list(b1+b2+b3~1), spar="m",
            start=list(params=c(0.0000003694342,419.2645,-0.00144125)),
            verbose=T)

co <- star.fit.1$coef
tx <- log(1+co[3]*x/co[2])/co[3]
xfold <- tx-floor(tx)
yfold <- y/(1+co[1]*x)

grid <- seq(0,1,len=100)
star.p.1 <- intervals(star.fit.1,newdata=data.frame(u=grid),
    terms=list(f1=matrix(c(1,1,1,1,0,0,0,1),ncol=4,byrow=T)))
postscript(file="star2.ps",height=2.2,width=4.4,horizontal=F,pointsize=10)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(grid, star.p.1$f1$fit[,1],xlab="x",ylab=expression(f[1]),
 type="n",ylim=c(3.5,10)) #ylim=c(5,9))
points(xfold, yfold, pch=".", col=gray(0:8/8)[3], cex=.5)
polygon(c(grid,rev(grid)),
        c(star.p.1$f1$fit[,1]-2*star.p.1$f1$pstd[,1],
          rev(star.p.1$f1$fit[,1]+2*star.p.1$f1$pstd[,1])),
          col=gray(0:8/8)[7], border=NA)
lines(grid, star.p.1$f1$fit[,1])
mtext("(a)")

plot(grid, star.p.1$f1$fit[,2],xlab="x",type="n",ylim=c(-.01,.01),
 ylab=expression(paste(P[1],f[1])))
mtext("(b)")
polygon(c(grid,rev(grid)),
        c(star.p.1$f1$fit[,2]-2*star.p.1$f1$pstd[,2],
          rev(star.p.1$f1$fit[,2]+2*star.p.1$f1$pstd[,2])),
        col=gray(0:8/8)[7], border=NA)
lines(grid, star.p.1$f1$fit[,2])
dev.off()



# nonparametric amplitude, shape using sin, initial parameters are
# based on the folded plot
star.fit.2 <- snr(y~b1+exp(f2(x))*sin(2*pi*(b2+log(1+b4*x/b3)/b4)),
            func=list(f2(u)~list(~u,cubic2(u))),
            params=list(b1+b2+b3+b4~1), spar="m",
            start=list(params=c(7.1726,.4215,419.2645,-0.00144125),
                       f=list(f2=log(diff(range(star.p.1$f1$fit[,1]))))),
            verbose=T, control=list(prec.out=.001,
            rkpk.control=list(limnla=c(12,14))))

co <- star.fit.2$coef
f4ini <- co[2]+log(1+co[4]*x/co[3])/co[4]
star.fit.3 <- snr(y~b+exp(f3(x))*sin(2*pi*f4(x)),
            func=list(f3(u)+f4(u)~list(~u,cubic2(u))),
            params=list(b~1), spar="m",
            start=list(params=c(7.1726),
                       f=list(f3=star.fit.2$funcFitted,f4=f4ini)),
            verbose=T, control=list(prec.out=.001,
            rkpk.control=list(limnla=c(12,14))))

postscript(file="star1.ps",height=3,width=4.4,horizontal=F,pointsize=10)
plot(x, y, xlab="time (day)", ylab="magnitude", type="p",pch=".")
lines(x, star.fit.3$fitted)
dev.off()

# estimate amplitudes within each period
tx1 <- function(x,par) {
 a <- par[1]
 b <- par[2]
 ifelse(b==0,tx <- x/a,tx <- log(1+b*x/a)/b)
 return(tx)
}

co <- star.fit.1$coef
nfold <- as.integer(tx1(x,co[2:3]))
d <- xm <- NULL
for (i in unique(nfold)) {
 ind <- nfold==i
 if (sum(ind)>5) {
  yaa <- y[ind]
  xaa <- xfold[ind]
  tmp <- lm(yaa~sin(2*pi*xaa)+cos(2*pi*xaa))$coef
  d <- c(d, log(tmp[2]**2+tmp[3]**2)/2)
  xm <- c(xm, median(x[ind]))
 }
}

# estimate periods
source("pulini.R")
tmp <- pulini(x,y,alpha=.4,method="CLUSTER")
pd <- mpd <- NULL
for (i in 2:length(tmp)) {
  pd <- c(pd, tmp[i]-tmp[i-1])
  mpd <- c(mpd, (tmp[i]+tmp[i-1])/2)
}

grid <- seq(min(x),max(x),len=100)
star.p.2 <- intervals(star.fit.2,newdata=data.frame(u=grid))
star.p.3 <- intervals(star.fit.3,newdata=data.frame(u=grid))
df4 <- diff(star.p.3$f4$fit[,1])/(grid[2]-grid[1])
amp.1 <- log((1+star.fit.1$coef[1]*grid)*diff(range(star.p.1$f1$fit[,1]))/2)


postscript(file="star3.ps",height=4,width=4.4,horizontal=F,pointsize=9)
par(mfrow=c(2,2),mgp=c(2,1,0), mar=c(3,3,2,1)+.1)

plot(grid, star.p.2$f2$fit[,1],xlab="time (day)",ylab=expression(f[2]),
 ylim=c(0,1),type="n")
mtext("(a)")
polygon(c(grid,rev(grid)),
        c(star.p.2$f2$fit[,1]-2*star.p.2$f2$pstd,
         rev(star.p.2$f2$fit[,1]+2*star.p.2$f2$pstd)),
        col=gray(0:8/8)[8], border=NA)
lines(grid, star.p.2$f2$fit[,1])
lines(grid, amp.1, lty=2)
points(xm, d, pch="o")

plot(grid, star.p.3$f3$fit[,1],xlab="time (day)",ylab=expression(f[3]),
 ylim=c(0,1),type="n")
mtext("(b)")
polygon(c(grid,rev(grid)),
        c(star.p.3$f3$fit[,1]-2*star.p.3$f3$pstd,
         rev(star.p.3$f3$fit[,1]+2*star.p.3$f3$pstd)),
        col=gray(0:8/8)[8], border=NA)
lines(grid, star.p.3$f3$fit[,1])
points(xm, d, pch="o")

plot(grid, star.p.3$f4$fit[,1],xlab="time (day)",ylab=expression(f[4]),type="n")
mtext("(c)")
polygon(c(grid,rev(grid)),
        c(star.p.3$f4$fit[,1]-2*star.p.3$f4$pstd,
          rev(star.p.3$f4$fit[,1]+2*star.p.3$f4$pstd)),
        col=gray(0:8/8)[8], border=NA)
lines(grid, star.p.3$f4$fit[,1])

plot((grid[1:99]+grid[2:100])/2, 1/df4, xlab="time (day)",
 ylab="period",type="l",ylim=c(200,600))
mtext("(d)")
points(mpd, pd, pch="o")

dev.off()

n <- length(y)
rss <- c(sum((y-star.fit.1$fitted)**2),
         sum((y-star.fit.2$fitted)**2),
         sum((y-star.fit.3$fitted)**2))/n
df <- c(star.fit.1$df$f+star.fit.1$df$para,
        star.fit.2$df$f+star.fit.2$df$para,
        star.fit.3$df$f+star.fit.3$df$para)
gcv <- rss/(1-df/n)**2
aic <- n*log(rss)+2*df
bic <- n*log(rss)+log(n)*df
print(round(rbind(aic, bic, gcv),4))


