# Code for Stratford data

library(assist)
library(lattice)
data(Stratford); attach(Stratford)
n <- 73
K <- (n-1)/2

strip.math <- function(which.given, which.panel, var.name, factor.levels, ...) {
   vn <- var.name[which.given]
   fl <- factor.levels
   expr <- paste(vn, "==", fl, collapse = ",")  
   expr <- paste("expression(", expr, ")", sep = "")  
   fl <- eval(parse(text=expr))
   strip.default(which.given, which.panel, vn, fl, ...) 
}

trellis.device(postscript, file="stratford1.ps", horizontal=F, 
width=4.6, height=3.5) 
par(mar=c(3,3,1,1)+0.1,oma=c(0,0,0,0))
selfit <- rep(mean(y),n)
selind <- c(.001,.0001,.00001,.000001,10^{-30})*n
for (i in 1:length(selind)) 
  selfit <- c(selfit,ssr(y~1,periodic(x),limnla=log10(selind[i]))$fit)

lb <- c("infinity",.001,.0001,.00001,.000001,0)
tmp1 <- data.frame(time=rep(x,6),temp=rep(y,6),lambda=rep(lb,rep(n,6)))
tmp2 <- data.frame(time=rep(x,6),temp=selfit,lambda=rep(lb,rep(n,6)))
tmp1$rowType <- rep("original",6*n)
tmp2$rowType <- rep("fit",6*n)
tmp <- rbind(tmp1,tmp2)
tmp$lambda <- factor(tmp$lambda,levels=c(.00001,.000001,0,"infinity",.001,.0001))

xyplot(temp~time|lambda, data=tmp, subscripts=T, groups=tmp$rowType,
       strip.names=c(F,F), layout=c(3,2),
       par.settings=list(fontsize=list(text=10,points=6)),
       strip = strip.math, par.strip.text=list(cex=0.7),
       scales=list(cex=.5),
       xlab=list("time",cex=.7),
       ylab=list("temperature (Fahrenheit)",cex=.7),
       panel=function(x,y,subscripts,groups,...){
             orig <- groups[subscripts]=="original"
             fit <- groups[subscripts]=="fit"
             panel.xyplot(x[orig], y[orig], pch=".")
             panel.superpose(x[fit], y[fit], subscripts[fit], groups, type="l")})
dev.off()

postscript(file="lowpass.ps",height=2,width=3,horizontal=F,pointsize=8)
par(mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)
lambda <- c(100,10^{-3},10^{-4},10^{-5},10^{-6},10^{-30})
plot(0:36,rep(1,37),type="n",ylim=c(0,1),xlab="frequency",ylab="weights")
for (i in 1:length(lambda)) {
   w <- 1/(1+lambda[i]*(2*pi*(0:36))**4)
   lines(0:36,w)
}
dev.off()

postscript(file="dfrss.ps",height=2,width=4.4,horizontal=F,pointsize=7)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)

X <- rep(1,n)
rss1 <- NULL
for (i in 0:K) {
  if (i>0) X <- cbind(X,sqrt(2)*sin(2*pi*i*x),sqrt(2)*cos(2*pi*i*x))
  rss1 <- c(rss1,sum((y-X%*%t(X)%*%y/n)**2))
}

lambda <- seq(-12,0,len=100)
df <- rss2 <- NULL
for (i in 1:100) {
  fit <- ssr(y~1,periodic(x),
    limnla=c(log10(n)+lambda[i],log10(n)+lambda[i]))
  df <- c(df,sum(diag(hat.ssr(fit))))
  rss2 <- c(rss2,sum(fit$resi**2))
}
plot(lambda,df,xlab=expression(paste(log,scriptscriptstyle(10),
     "(",lambda,")")),ylab="degrees of freedom",type="l")
mtext("(a)")

plot(2*(0:36)+1,sqrt(rss1/n),type="n",
     xlab="degrees of freedom",ylab="square root of RSS")
points(2*(0:36)+1,sqrt(rss1/n),cex=.8,pch="o")
lines(df,sqrt(rss2/n))
mtext("(b)")

dev.off()


