two.bond <- function(price, payment, time, name, type, spar="m", 
 limnla=c(-3,6), prec=1.e-6, maxit=20) {
 # pre-processing the data
 # the data in each group has to be sorted by the name
 group1 <- as.vector(table(name[type=="govt"]))
 y1 <- price[type=="govt"][cumsum(group1)]
 time1 <- time[type=="govt"]  
 n1.time <- length(time1)
 payment1 <- payment[type=="govt"]
 group2 <- as.vector(table(name[type=="ge"]))
 y2 <- price[type=="ge"][cumsum(group2)]
 time2 <- time[type=="ge"]
 n2.time <- length(time2) 
 payment2 <- payment[type=="ge"]
 y <- c(y1, y2)
 group <- c(group1, group2)
 payment <- c(payment1, payment2)
 error <- 0

 # create points for Gaussian quadrature, 3 points in each interval 
 s.time1 <- sort(time1)
 x1.y1 <- c(0, s.time1[-n1.time])
 x2.y1 <- s.time1-x1.y1
 org.ord1 <- match(1:n1.time, (1:n1.time)[order(time1)])
 q.time1 <- as.vector(rep(1,3)%o%x1.y1+c(0.1127017,0.5,0.8872983)%o%x2.y1)
 s.time2 <- sort(time2)
 x1.y2 <- c(0, s.time2[-n2.time])
 x2.y2 <- s.time2-x1.y2
 org.ord2 <- match(1:n2.time, (1:n2.time)[order(time2)])
 q.time2 <- as.vector(rep(1,3)%o%x1.y2+c(0.1127017,0.5,0.8872983)%o%x2.y2)

 # initial values for f
 f10 <- function(x) rep(0.04, length(x))    
 f20 <- function(x) rep(0.01, length(x))
 f1.val1 <- f10(q.time1)
 f1.val2 <- f10(q.time2)
 f1.old <- c(f1.val1,f1.val2)  
 f2.val2 <- f20(q.time2)
 f2.old <- f2.val2

 # create s and q matrices
 S1 <- cbind(time1,time1*time1/2.0)               
 S2 <- cbind(time2,time2*time2/2.0)
 L1 <- int1(s.time1,rep(1,3*length(s.time1)))[org.ord1,org.ord1]
 L2 <- int1(s.time2,rep(1,3*length(s.time2)))[org.ord2,org.ord2]
 L12 <- int2(s.time1,s.time2,rep(1,3*length(s.time1)), rep(1,3*length(s.time2)))
 L12 <- L12[org.ord1,org.ord2]
 Lambda <- rbind(cbind(L1,L12),cbind(t(L12),L2))
 L1int <- int.f(s.time1,c(q.time1,q.time2),rep(1,3*length(s.time1)))[org.ord1,]
 L2int <- int.f(s.time2,c(q.time1,q.time2),rep(1,3*length(s.time2)))[org.ord2,]        
 Lint <- rbind(L1int,L2int)
 L2int2 <- int.f(s.time2,q.time2,rep(1,3*length(s.time2)))[org.ord2,]

 # begin iteration
 iter <- cover <- 1    
 repeat {
  # update f1
  f1int1 <- int.s(f1.val1,s.time1)[org.ord1]
  f1int2 <- int.s(f1.val2,s.time2)[org.ord2]
  f2int2 <- int.s(f2.val2,s.time2)[org.ord2]
  X <- diagComp(matrix(payment*exp(-c(f1int1,f1int2+f2int2)),nrow=1),group)
  ytilde1 <- X%*%(1+c(f1int1,f1int2))-y
  T <- X%*%rbind(S1,S2)
  Q <- X%*%Lambda%*%t(X)
  fit1 <- try(ssr(ytilde1~T-1, Q, spar=spar, limnla=limnla))
  if (class(fit1)=="try-error") {error=1; break}
  if (class(fit1)!="try-error") {
   dd <- fit1$coef$d; cc <- fit1$coef$c
   f1.val <- as.vector((cc%*%X)%*%Lint+dd[1]+dd[2]*c(q.time1,q.time2))        
   f1.val1 <- f1.val[1:(3*n1.time)]
   f1.val2 <- f1.val[-(1:(3*n1.time))]            
  }

  # update f2
  f1int2 <- int.s(f1.val2,s.time2)[org.ord2]
  X2 <- diagComp(matrix(payment2*exp(-f1int2-f2int2),nrow=1),group2)                
  ytilde2 <- X2%*%(1+f2int2)-y2
  T2 <- X2%*%S2
  Q22 <- X2%*%L2%*%t(X2)
  fit2 <- try(ssr(ytilde2~T2-1, Q22, spar=spar, limnla=limnla))
  if (class(fit2)=="try-error") {error=1; break}
  if (class(fit2)!="try-error") {
   dd <- fit2$coef$d; cc <- fit2$coef$c
   f2.val2 <- as.vector((cc%*%X2)%*%L2int2+dd[1]+dd[2]*q.time2)
  }
  cover <- mean((c(f1.val1,f1.val2,f2.val2)-c(f1.old, f2.old))^2)
  if(cover<prec || iter>maxit) break
  iter<- iter + 1                      
  f1.old <- c(f1.val1,f1.val2)
  f2.old <- f2.val2
 }
 tmp1 <- -int.s(f1.val1,s.time1)[org.ord1]
 tmp2 <- -int.s(f1.val2+f2.val2,s.time2)[org.ord2]
 yhat <- apply(diagComp(matrix(payment*exp(c(tmp1,tmp2)),nrow=1),group),1,sum)
 sigma <- NA
 if (error==0) sigma <- sqrt(sum((y-yhat)^2)/(length(y)-fit1$df-fit2$df))
 list(fit1=fit1, fit2=fit2, iter=c(iter, cover, error), call=match.call(), 
      f.val=list(f1=f1.val1,f2=f1.val2+f2.val2), 
      f2.val=f2.val2, q.time=list(q.time1=q.time1,q.time2=q.time2), 
      dc=list(dc1=exp(tmp1),dc2=exp(tmp2)),
      y=list(y=y,yhat=yhat), sigma=sigma)
}    


