# Code for weight loss data

library(assist)
library(MASS)
attach(wtloss)
y <- Weight
x <- Days

# fit the non-linear regression model
weight.nls <- nls(y~b1+b2*exp(-b3*x),
                  start=list(b1=81,b2=102,b3=.005))

# fit exponential spline
r <- coef(weight.nls)[3]
weight.exp <- ssr(y~exp(-r*x),rk=lspline(r*x,type="exp"))

# fit exponential spline with transformed x
tx <- 1-exp(-r*x)
weight.exp1 <- ssr(y~tx,rk=cubic2(tx))
weight.exp2 <- ssr(y~tx,rk=cubic(tx),scale=T)

# cubic fits 
weight.cub1 <- ssr(y~x,cubic2(x))
weight.cub <- ssr(y~x,cubic(x),scale=T)

weight.p.nls <- predict(weight.nls)
weight.p.exp <- predict(weight.exp)
weight.p.cub <- predict(weight.cub)
d.exp <- weight.p.exp$fit-weight.p.nls
d.cub <- weight.p.cub$fit-weight.p.nls

postscript(file="weight1.ps",height=2,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(x,y,ylab="Weight (kg)",xlab="Days",type="n")
points(x,y,pch="o",cex=.5)
lines(x,predict(weight.nls),lty=3)
lines(x,weight.exp$fit,lty=1)
lines(x,weight.cub1$fit,lty=2)
mtext("(a)")

plot(x,d.exp,ylab="",xlab="Days",type="n",ylim=c(-1.5,1.5))
lines(x,d.exp)
lines(x,d.exp-1.96*weight.p.exp$pstd,col=gray(0:8/8)[7])
lines(x,d.exp+1.96*weight.p.exp$pstd,col=gray(0:8/8)[7])
lines(x,d.cub,lty=2)
lines(x,d.cub-1.96*weight.p.cub$pstd,col=gray(0:8/8)[7],lty=2)
lines(x,d.cub+1.96*weight.p.cub$pstd,col=gray(0:8/8)[7],lty=2)
mtext("(b)")
dev.off()

# estimate r
gcv.fun <- function(r) {
 ssr(y~exp(-r*x),rk=lspline(r*x,type="exp"))$rkpk.obj$score
}
nlm(gcv.fun,.001)$estimate
#optim(.004, tmp.fun, lower=0, method="L-BFGS-B")


